/*
 * Pasek TVN24 v0.2.8
 *
 * Copyright (C) 2007 Adam Klobukowski
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <panel-applet.h>
#include <panel-applet-gconf.h>
#include <gtk/gtk.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkimage.h>
#include <gtk/gtkbox.h>
#include <gtk/gtkdrawingarea.h>
#include <gtk/gtkspinbutton.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtkfontbutton.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkhbox.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libgnome/libgnome.h>

#include <libxml/parser.h>
#include <libxml/tree.h>

#include "tvn24.h"

/* Menu kontekstowe */
static const gchar Context_menu_xml [] =
    "<popup name=\"button3\">\n"
    "   <menuitem name=\"RSS Item\" "
    "             verb=\"OpenTVN24RSS\" "
    "           _label=\"Otwórz RSS TVN24\"\n"
    "          pixtype=\"stock\" "
    "          pixname=\"gtk-info\"/>\n"
    "   <menuitem name=\"PropertiesItem\" "
    "             verb=\"Preferences\" "
    "           _label=\"_Ustawienia paska...\"\n"
    "          pixtype=\"stock\" "
    "          pixname=\"gtk-properties\"/>\n"
    "   <menuitem name=\"About Item\" "
    "             verb=\"About\" "
    "           _label=\"_O pasku...\"\n"
    "          pixtype=\"stock\" "
    "          pixname=\"gtk-about\"/>\n"
    "</popup>\n";

static const BonoboUIVerb menu_verbs [] = {
        BONOBO_UI_VERB ((void*)"Preferences", (void*)display_preferences_dialog),
        BONOBO_UI_VERB ((void*)"About", (void*)display_about_dialog),
        BONOBO_UI_VERB ((void*)"OpenTVN24RSS", (void*)open_tvn24_rss),
        BONOBO_UI_VERB_END
};

#ifndef HAVE_GDK_COLOR_TO_STRING
/* Wsteczna kompatybilnosc z gtk < 2.11 */
gchar* gdk_color_to_string (const GdkColor *color)
{
    return g_strdup_printf ("#%04x%04x%04x", color->red, color->green, color->blue);
} 
#endif

/* przycisk domyślnych ustawień */
void on_domyslnebutton_clicked (GtkButton *button, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    GtkWidget *dialog = NULL;
    gchar* color = NULL;
    gint ival = 0;

    dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, _("Czy napewno przywrócić domyślne ustawienia?"));
    ival = gtk_dialog_run (GTK_DIALOG (dialog));
    
    gtk_widget_destroy (dialog);

    if (ival != GTK_RESPONSE_YES) return;

/* zmiana w strukturze */
    g_free (pa->init_text);
    pa->init_text = g_strdup (INIT_TEXT);
    g_free (pa->init_font);
    pa->init_font = g_strdup (START_FONT);
    g_free(pa->kraj_font);
    pa->kraj_font = g_strdup (START_FONT);
    g_free (pa->swiat_font);
    pa->swiat_font = g_strdup (START_FONT);
    g_free (pa->pilne_font);
    pa->pilne_font = g_strdup (START_FONT);

    pa->scroll_size = 300;

    pa->speed = 16;

    pa->skok = 0;
    pa->skok_t = 0;
    pa->init_kierunek = 0;
    pa->pilne_kierunek = 0;
    pa->kraj_kierunek = 0;
    pa->swiat_kierunek = 0;
    pa->rozciagnij = 0;
    pa->przeciagarka = 0;

    gdk_color_parse (FG_KRAJ, pa->kraj_fg_color);
    gdk_color_parse (BG_KRAJ, pa->kraj_bg_color);
    gdk_color_parse (FG_SWIAT, pa->swiat_fg_color);
    gdk_color_parse (BG_SWIAT, pa->swiat_bg_color);
    gdk_color_parse (FG_PILNE, pa->pilne_fg_color);
    gdk_color_parse (BG_PILNE, pa->pilne_bg_color);
    gdk_color_parse (FG_INIT, pa->init_fg_color);
    gdk_color_parse (BG_INIT, pa->init_bg_color);
    gdk_color_parse (BG_TLO, pa->tlo_color);

/* zapisanie zmian w gconfie */
    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "przeciagarka", pa->przeciagarka, NULL);
    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "rozciagnij", pa->rozciagnij, NULL);
    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "init_text", pa->init_text, NULL);

    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "init_kierunek", pa->init_kierunek, NULL);
    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "pilne_kierunek", pa->pilne_kierunek, NULL);
    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "kraj_kierunek", pa->kraj_kierunek, NULL);
    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "swiat_kierunek", pa->swiat_kierunek, NULL);
    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "speed", pa->speed, NULL);
    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "skok", pa->skok, NULL);

    color = gdk_color_to_string (pa->pilne_bg_color);
    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "tlo_color", color, NULL);
    g_free (color);

    color = gdk_color_to_string (pa->init_bg_color);
    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "init_bg_color", color, NULL);
    g_free (color);

    color = gdk_color_to_string (pa->pilne_bg_color);
    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "pilne_bg_color", color, NULL);
    g_free (color);

    color = gdk_color_to_string (pa->kraj_bg_color);
    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "krak_bg_color", color, NULL);
    g_free (color);

    color = gdk_color_to_string (pa->swiat_bg_color);
    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "swiat_bg_color", color, NULL);
    g_free (color);

    color = gdk_color_to_string (pa->init_fg_color);
    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "init_fg_color", color, NULL);
    g_free (color);

    color = gdk_color_to_string (pa->pilne_fg_color);
    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "pilne_fg_color", color, NULL);
    g_free (color);

    color = gdk_color_to_string (pa->kraj_fg_color);
    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "krak_fg_color", color, NULL);
    g_free (color);

    color = gdk_color_to_string (pa->swiat_fg_color);
    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "swiat_fg_color", color, NULL);
    g_free (color);

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "init_font", pa->init_font, NULL);
    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "pilne_font", pa->pilne_font, NULL);
    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "kraj_font", pa->kraj_font, NULL);
    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "swiat_font", pa->swiat_font, NULL);

    if (pa->entries->len > 0)
    {
    	if (((struct CrawlEntry*)g_ptr_array_index (pa->entries, 0))->et == INIT)
    	{
    	    g_free (((struct CrawlEntry*)g_ptr_array_index (pa->entries, 0))->text);
    	    ((struct CrawlEntry*)g_ptr_array_index (pa->entries, 0))->text = g_strdup (pa->init_text);
    	}
    }

/* zmiana w oknie preferencji */
    preferences_set (pa);

/* zmiana timera */
    g_source_remove (pa->timeout);

    pa->timeout = g_timeout_add (pa->speed, draw_timer, (gpointer) pa);

/* zmiany paneli */
    ival = PANEL_APPLET_FLAGS_NONE;
    if (pa->rozciagnij) ival |= PANEL_APPLET_EXPAND_MAJOR;
    if (pa->przeciagarka) ival |= PANEL_APPLET_HAS_HANDLE;
    
    panel_applet_set_flags (PANEL_APPLET (pa->papplet), ival);

/* nowy layout */    
    prepare_layouts (pa->entries, pa);
}

/* checkbox od przeciągarki */
void on_przeciagarka_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    gint ival = 0;

    pa->przeciagarka = (gint)gtk_toggle_button_get_active (togglebutton);
    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "przeciagarka", pa->przeciagarka, NULL);

/* prawdopodobnie błąd w libpanelapplet */
    ival = PANEL_APPLET_FLAGS_NONE;
    if (!pa->rozciagnij) ival |= PANEL_APPLET_EXPAND_MAJOR;
    if (pa->przeciagarka) ival |= PANEL_APPLET_HAS_HANDLE;

    panel_applet_set_flags (PANEL_APPLET (pa->papplet), ival);

    gtk_main_iteration_do (TRUE);
    while (gtk_events_pending ()) gtk_main_iteration_do (FALSE);
/* koniec obejścia błędu */

    ival = PANEL_APPLET_FLAGS_NONE;
    if (pa->rozciagnij) ival |= PANEL_APPLET_EXPAND_MAJOR;
    if (pa->przeciagarka) ival |= PANEL_APPLET_HAS_HANDLE;
    
    panel_applet_set_flags (PANEL_APPLET (pa->papplet), ival);
}

/* checkbox od rozmiaru */
void on_wypelnij_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    gint ival = 0;

    pa->rozciagnij = (gint)gtk_toggle_button_get_active (togglebutton);
    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "rozciagnij", pa->rozciagnij, NULL);


    ival = PANEL_APPLET_FLAGS_NONE;
    if (pa->rozciagnij) ival |= PANEL_APPLET_EXPAND_MAJOR;
    if (pa->przeciagarka) ival |= PANEL_APPLET_HAS_HANDLE;

    panel_applet_set_flags (PANEL_APPLET (pa->papplet), ival);
}

/* zmiana tekstu poczatkowego */
void on_inittext_changed (GtkEditable *editable, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    g_free (pa->init_text);
    pa->init_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (editable)));

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "init_text", pa->init_text, NULL);

    if (pa->entries->len > 0)
    {
    	if (((struct CrawlEntry*)g_ptr_array_index (pa->entries, 0))->et == INIT)
    	{
    	    g_free (((struct CrawlEntry*)g_ptr_array_index (pa->entries, 0))->text);
    	    ((struct CrawlEntry*)g_ptr_array_index (pa->entries, 0))->text = g_strdup (pa->init_text);
    	}
    }
    
    prepare_layouts (pa->entries, pa);
}

/* zmiana kierunku KRAJ */
void on_krajkierunek_changed (GtkComboBox *combobox, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    pa->kraj_kierunek = gtk_combo_box_get_active (combobox);

    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "kraj_kierunek", pa->kraj_kierunek, NULL);

    prepare_layouts (pa->entries, pa);
}

/* zmiana kierunku SWIAT */
void on_swiatkierunek_changed (GtkComboBox *combobox, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    pa->swiat_kierunek = gtk_combo_box_get_active (combobox);

    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "swiat_kierunek", pa->swiat_kierunek, NULL);

    prepare_layouts (pa->entries, pa);
}

/* zmiana kierunku INIT */
void on_initkierunek_changed (GtkComboBox *combobox, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    pa->init_kierunek = gtk_combo_box_get_active (combobox);

    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "init_kierunek", pa->init_kierunek, NULL);

    prepare_layouts (pa->entries, pa);
}

/* zmiana kierunku PILNE */
void on_pilnekierunek_changed (GtkComboBox *combobox, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    pa->pilne_kierunek = gtk_combo_box_get_active (combobox);

    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "pilne_kierunek", pa->pilne_kierunek, NULL);

    prepare_layouts (pa->entries, pa);
}

/* Obsluga spinbutona w oknie preferencji */
void on_szerokoscspibutton_value_changed (GtkSpinButton *spinbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    pa->scroll_size = gtk_spin_button_get_value_as_int (spinbutton);

    if (pa->porient == PANEL_APPLET_ORIENT_UP || pa->porient == PANEL_APPLET_ORIENT_DOWN) gtk_widget_set_size_request (pa->drawing_area, pa->scroll_size, pa->panel_size);
    else gtk_widget_set_size_request (pa->drawing_area, pa->panel_size, pa->scroll_size);

    reset_pixmap (pa);
}

/* zamkniecie okna preferencji */
void on_zamknijbutton_clicked (GtkButton *button, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    gtk_widget_destroy (pa->preferences);
    pa->preferences = NULL;
}

/* a to jak zamyka sie okno preferencji X */
gboolean on_preferences_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    gtk_widget_destroy (pa->preferences);
    pa->preferences = NULL;

    return FALSE;
}

/* zmiana prędkości crawla */
gboolean on_szybkoschscale_change_value (GtkRange *range, GtkScrollType scroll, gdouble value, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    pa->speed = 33 - (gint)value;
    if (pa->speed < 2) pa->speed = 2; /* poniżej 2 robi się za szybko */

    g_source_remove (pa->timeout);

    pa->timeout = g_timeout_add (pa->speed, draw_timer, (gpointer) pa);

    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "speed", pa->speed, NULL);

    return FALSE;
}

/* zmiana prędkości crawla */
gboolean on_skokhscale_change_value (GtkRange *range, GtkScrollType scroll, gdouble value, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    pa->skok = (gint)value;

    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "skok", pa->skok, NULL);

    return FALSE;
}

/* zmiana koloru tła */
void on_tlocolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    gchar* color = NULL;

    gtk_color_button_get_color (colorbutton, pa->tlo_color);

    color = gdk_color_to_string (pa->tlo_color);

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "tlo_color", color, NULL);

    g_free (color);

    prepare_layouts (pa->entries, pa);
}

/* zmiana koloru tła PILNE */
void on_pilnebgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    gchar* color = NULL;

    gtk_color_button_get_color (colorbutton, pa->pilne_bg_color);

    color = gdk_color_to_string (pa->pilne_bg_color);

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "pilne_bg_color", color, NULL);

    g_free (color);

    prepare_layouts (pa->entries, pa);
}

/* zmiana koloru tła Start */
void on_initbgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    gchar* color = NULL;

    gtk_color_button_get_color (colorbutton, pa->init_bg_color);

    color = gdk_color_to_string (pa->init_bg_color);

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "init_bg_color", color, NULL);

    g_free (color);

    prepare_layouts (pa->entries, pa);
}

/* zmiana koloru tła KRAJ */
void on_krajbgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    gchar* color = NULL;

    gtk_color_button_get_color (colorbutton, pa->kraj_bg_color);

    color = gdk_color_to_string (pa->kraj_bg_color);

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "kraj_bg_color", color, NULL);

    g_free (color);

    prepare_layouts (pa->entries, pa);
}

/* zmiana koloru tła SWIAT */
void on_swiatbgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    gchar* color = NULL;

    gtk_color_button_get_color (colorbutton, pa->swiat_bg_color);

    color = gdk_color_to_string (pa->swiat_bg_color);

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "swiat_bg_color", color, NULL);

    g_free (color);

    prepare_layouts (pa->entries, pa);
}

/* zmiana koloru tekstu PILNE */
void on_pilnefgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    gchar* color = NULL;

    gtk_color_button_get_color (colorbutton, pa->pilne_fg_color);

    color = gdk_color_to_string (pa->pilne_fg_color);

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "pilne_fg_color", color, NULL);

    g_free (color);

    prepare_layouts (pa->entries, pa);
}

/* zmiana koloru tekstu Start */
void on_initfgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    gchar* color = NULL;

    gtk_color_button_get_color (colorbutton, pa->init_fg_color);

    color = gdk_color_to_string (pa->init_fg_color);

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "init_fg_color", color, NULL);

    g_free (color);

    prepare_layouts (pa->entries, pa);
}

/* zmiana koloru tekstu SWIAT */
void on_swiatfgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    gchar* color = NULL;

    gtk_color_button_get_color (colorbutton, pa->swiat_fg_color);

    color = gdk_color_to_string (pa->swiat_fg_color);

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "swiat_fg_color", color, NULL);

    g_free (color);

    prepare_layouts (pa->entries, pa);
}

/* zmiana czcionki PILNE */
void on_pilnefontbutton_font_set (GtkFontButton *fontbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    g_free (pa->pilne_font);
    pa->pilne_font = g_strdup (gtk_font_button_get_font_name (fontbutton));

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "pilne_font", pa->pilne_font, NULL);

    prepare_layouts (pa->entries, pa);
}

/* zmiana czcionki Start */
void on_initfontbutton_font_set (GtkFontButton *fontbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    g_free (pa->init_font);
    pa->init_font = g_strdup (gtk_font_button_get_font_name (fontbutton));

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "init_font", pa->init_font, NULL);

    prepare_layouts (pa->entries, pa);
}

/* zmiana czcionki KRAJ */
void on_krajfontbutton_font_set (GtkFontButton *fontbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    g_free (pa->kraj_font);
    pa->kraj_font = g_strdup (gtk_font_button_get_font_name (fontbutton));

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "kraj_font", pa->kraj_font, NULL);

    prepare_layouts (pa->entries, pa);
}

/* zmiana czcionki SWIAT */
void on_swiatfontbutton_font_set (GtkFontButton *fontbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    g_free (pa->swiat_font);
    pa->swiat_font = g_strdup (gtk_font_button_get_font_name (fontbutton));

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "swiat_font", pa->swiat_font, NULL);

    prepare_layouts (pa->entries, pa);
}

/* zmianaa koloru tekstu KRAJ */
void on_krajfgcolorbutton_color_set (GtkColorButton  *colorbutton, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    gchar* color = NULL;

    gtk_color_button_get_color (colorbutton, pa->kraj_fg_color);

    color = gdk_color_to_string (pa->kraj_fg_color);

    panel_applet_gconf_set_string (PANEL_APPLET (pa->papplet), "kraj_fg_color", color, NULL);

    g_free (color);

    prepare_layouts (pa->entries, pa);
}

/* Zmiana orientacji panelu */
void applet_orientation_change (PanelApplet *panel, gint arg1, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    pa->porient = panel_applet_get_orient(panel);
    pa->panel_size = panel_applet_get_size (panel);

    if (pa->porient == PANEL_APPLET_ORIENT_UP || pa->porient == PANEL_APPLET_ORIENT_DOWN) gtk_widget_set_size_request (pa->drawing_area, pa->scroll_size, pa->panel_size);
    else gtk_widget_set_size_request (pa->drawing_area, pa->panel_size, pa->scroll_size);

    reset_pixmap (pa);

    prepare_layouts (pa->entries, pa);
}

/* rekonifiguracja */
gboolean configure_event (GtkWidget *widget, GdkEventConfigure *event, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    panel_applet_gconf_set_int (PANEL_APPLET (pa->papplet), "size", pa->scroll_size, NULL);

    reset_pixmap (pa);

    prepare_layouts (pa->entries, pa);

    return TRUE;
}

/* odmalowanie */
gboolean expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    GdkGC *gc;

    gc = gdk_gc_new (pa->drawing_area->window);
    gdk_gc_set_rgb_bg_color (gc, pa->init_bg_color);
    gdk_gc_set_rgb_fg_color (gc, pa->init_bg_color);

    reset_pixmap (pa);

    draw (pa);

    gdk_gc_unref (gc);

    return FALSE;
}

/* realizacja przeciagania paska przy pomocy myszki i obsługa podwójnego kliknieciea */
gboolean drawable_on_button_press (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    gint o = 0; /* poprzednia pozycja scrolla*/
    gint x = 0; /* współrzędna X kursora */
    gint y = 0; /* współrzędna Y kursora */
    gint c = 0; /* początkowa istotna współrzędna kursora */
    gint t = 0; /* pozycja scrola */
    GdkModifierType mask;
    GdkDisplay* display = NULL;
    GdkCursor* cursor = NULL;

/* podwojne klikniecie */
    if (event->type == GDK_2BUTTON_PRESS)
    {
        open_tvn24_rss (NULL, NULL);
    }

/* przeciąganie */
    gdk_window_get_pointer (pa->drawing_area->window, &x, &y, &mask);

    if ((mask & GDK_BUTTON1_MASK) == GDK_BUTTON1_MASK)
    {

        if (pa->porient == PANEL_APPLET_ORIENT_UP || pa->porient == PANEL_APPLET_ORIENT_DOWN) c = x;
    	else c = y;

        t = pa->t;
    	o = pa->t;

    	g_source_remove (pa->timeout);

    	display = gdk_display_get_default ();
    	cursor = gdk_cursor_new_for_display (display, GDK_FLEUR);
    	gdk_window_set_cursor (pa->drawing_area->window, cursor);

        while ((mask & GDK_BUTTON1_MASK) == GDK_BUTTON1_MASK)
    	{
    	    gdk_window_get_pointer (pa->drawing_area->window, &x, &y, &mask);

    	    if (pa->porient == PANEL_APPLET_ORIENT_UP || pa->porient == PANEL_APPLET_ORIENT_DOWN) pa->t = t + (x - c);
    	    if (pa->porient == PANEL_APPLET_ORIENT_LEFT) pa->t = t + (y - c);
    	    if (pa->porient == PANEL_APPLET_ORIENT_RIGHT) pa->t = t + (c - y);

    	    if (o != pa->t) draw (pa);

    	    o = pa->t;
    	}

    	if (pa->t > pa->scroll_size) pa->t = pa->scroll_size;

        pa->timeout = g_timeout_add (pa->speed, draw_timer, (gpointer) pa);

    	gdk_window_set_cursor (pa->drawing_area->window, NULL);

    	gdk_cursor_unref (cursor);

    	return TRUE;
    }

    return FALSE;
}

/* wyszukanie widgeta(z Glade) */
GtkWidget* lookup_widget (GtkWidget *widget, const gchar *widget_name)
{
    GtkWidget *parent, *found_widget;

    for (;;)
    {
        if (GTK_IS_MENU (widget)) parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
        else parent = widget->parent;

        if (!parent) parent = (GtkWidget*) g_object_get_data (G_OBJECT (widget), "GladeParentKey");

        if (parent == NULL) break;

        widget = parent;
    }

    found_widget = (GtkWidget*) g_object_get_data (G_OBJECT (widget), widget_name);
    if (!found_widget) g_warning ("Widget not found: %s", widget_name);

    return found_widget;
}

/* stworzenie okna preferencji */
GtkWidget* create_preferences (PasekTVN24Private* pa)
{
    GtkWidget *preferences;
    GtkWidget *vbox2;
    GtkWidget *notebook;
    GtkWidget *table2;
    GtkWidget *przeciagarkacheckbox;
    GtkWidget *wypelnijcheckbox;
    GtkWidget *label34;
    GtkWidget *label11;
    GtkWidget *label35;
    GtkWidget *label2;
    GtkObject *szerokoscspinbutton_adj;
    GtkWidget *szerokoscspinbutton;
    GtkWidget *predkoschscale;
    GtkWidget *skokhscale;
    GtkWidget *tlocolorbutton;
    GtkWidget *label12;
    GtkWidget *table5;
    GtkWidget *label29;
    GtkWidget *initkierunek;
    GtkWidget *initfontbutton;
    GtkWidget *label25;
    GtkWidget *label28;
    GtkWidget *alignment10;
    GtkWidget *label26;
    GtkWidget *alignment11;
    GtkWidget *label27;
    GtkWidget *label36;
    GtkWidget *initfgcolorbutton;
    GtkWidget *initbgcolorbutton;
    GtkWidget *inittext;
    GtkWidget *label7;
    GtkWidget *table7;
    GtkWidget *pilnefontbutton;
    GtkWidget *pilnekierunek;
    GtkWidget *label30;
    GtkWidget *label33;
    GtkWidget *alignment12;
    GtkWidget *label31;
    GtkWidget *alignment13;
    GtkWidget *label32;
    GtkWidget *label5;
    GtkWidget *pilnebgcolorbutton;
    GtkWidget *pilnefgcolorbutton;
    GtkWidget *label8;
    GtkWidget *table4;
    GtkWidget *krajfontbutton;
    GtkWidget *label18;
    GtkWidget *alignment6;
    GtkWidget *label19;
    GtkWidget *alignment7;
    GtkWidget *label20;
    GtkWidget *label40;
    GtkWidget *krajkierunek;
    GtkWidget *krajbgcolorbutton;
    GtkWidget *krajfgcolorbutton;
    GtkWidget *label3;
    GtkWidget *labela;
    GtkWidget *table6;
    GtkWidget *swiatkierunek;
    GtkWidget *swiatfontbutton;
    GtkWidget *label21;
    GtkWidget *label22;
    GtkWidget *alignment8;
    GtkWidget *label23;
    GtkWidget *alignment9;
    GtkWidget *label24;
    GtkWidget *label41;
    GtkWidget *swiatfgcolorbutton;
    GtkWidget *swiatbgcolorbutton;
    GtkWidget *label4;
    GtkWidget *hbox2;
    GtkWidget *domyslnebutton;
    GtkWidget *zamknijbutton;
    GtkWidget *alignment1;
    GtkWidget *hbox1;
    GtkWidget *image1;
    GtkWidget *labelb;

    preferences = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (preferences), 5);
    gtk_window_set_title (GTK_WINDOW (preferences), _("Ustawienia paska"));
    gtk_window_set_position (GTK_WINDOW (preferences), GTK_WIN_POS_CENTER_ALWAYS);
    gtk_window_set_resizable (GTK_WINDOW (preferences), FALSE);
    gtk_window_set_skip_taskbar_hint (GTK_WINDOW (preferences), TRUE);
    gtk_window_set_skip_pager_hint (GTK_WINDOW (preferences), TRUE);
    gtk_window_set_type_hint (GTK_WINDOW (preferences), GDK_WINDOW_TYPE_HINT_DIALOG);

    vbox2 = gtk_vbox_new (FALSE, 12);
    gtk_widget_show (vbox2);
    gtk_container_add (GTK_CONTAINER (preferences), vbox2);
    gtk_container_set_border_width (GTK_CONTAINER (vbox2), 6);

    notebook = gtk_notebook_new ();
    gtk_widget_show (notebook);
    gtk_box_pack_start (GTK_BOX (vbox2), notebook, TRUE, TRUE, 0);

    table2 = gtk_table_new (6, 2, FALSE);
    gtk_widget_show (table2);
    gtk_container_add (GTK_CONTAINER (notebook), table2);
    gtk_container_set_border_width (GTK_CONTAINER (table2), 6);
    gtk_table_set_row_spacings (GTK_TABLE (table2), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table2), 12);

    przeciagarkacheckbox = gtk_check_button_new_with_mnemonic (_("poka\305\274 przeci\304\205gark\304\231"));
    gtk_widget_show (przeciagarkacheckbox);
    gtk_table_attach (GTK_TABLE (table2), przeciagarkacheckbox, 1, 2, 2, 3, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

    wypelnijcheckbox = gtk_check_button_new_with_mnemonic (_("wype\305\202nij ca\305\202e dost\304\231pne miejsce"));
    gtk_widget_show (wypelnijcheckbox);
    gtk_table_attach (GTK_TABLE (table2), wypelnijcheckbox, 1, 2, 1, 2, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

    label34 = gtk_label_new (_("Kolor t\305\202a:"));
    gtk_widget_show (label34);
    gtk_table_attach (GTK_TABLE (table2), label34, 0, 1, 5, 6, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_EXPAND), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label34), 0, 0.5);

    label11 = gtk_label_new (_("Pr\304\231dko\305\233\304\207 presuwania:"));
    gtk_widget_show (label11);
    gtk_table_attach (GTK_TABLE (table2), label11, 0, 1, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_EXPAND), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label11), 0, 0.5);

    label35 = gtk_label_new (_("Skokowo\305\233\304\207 przesuwania:"));
    gtk_widget_show (label35);
    gtk_table_attach (GTK_TABLE (table2), label35, 0, 1, 4, 5, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_EXPAND), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label35), 0, 0.5);

    label2 = gtk_label_new (_("Minimalna szeroko\305\233\304\207 paska:"));
    gtk_widget_show (label2);
    gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_EXPAND), 0, 0);
    gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_RIGHT);
    gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

    szerokoscspinbutton_adj = gtk_adjustment_new (300, 20, 9999, 1, 10, 10);
    szerokoscspinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (szerokoscspinbutton_adj), 1, 0);
    gtk_widget_show (szerokoscspinbutton);
    gtk_table_attach (GTK_TABLE (table2), szerokoscspinbutton, 1, 2, 0, 1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_EXPAND), 0, 0);
    gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (szerokoscspinbutton), TRUE);

    predkoschscale = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (16.0396, 1, 32, 0, 0, 0)));
    gtk_widget_show (predkoschscale);
    gtk_table_attach (GTK_TABLE (table2), predkoschscale, 1, 2, 3, 4, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_EXPAND), 0, 0);
    gtk_widget_set_size_request (predkoschscale, 100, -1);
    gtk_scale_set_draw_value (GTK_SCALE (predkoschscale), FALSE);
    gtk_scale_set_value_pos (GTK_SCALE (predkoschscale), GTK_POS_LEFT);
    gtk_scale_set_digits (GTK_SCALE (predkoschscale), 0);

    skokhscale = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 100, 1, 10, 10)));
    gtk_widget_show (skokhscale);
    gtk_table_attach (GTK_TABLE (table2), skokhscale, 1, 2, 4, 5, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_EXPAND), 0, 0);
    gtk_scale_set_draw_value (GTK_SCALE (skokhscale), FALSE);
    gtk_scale_set_digits (GTK_SCALE (skokhscale), 0);

    tlocolorbutton = gtk_color_button_new ();
    gtk_widget_show (tlocolorbutton);
    gtk_table_attach (GTK_TABLE (table2), tlocolorbutton, 1, 2, 5, 6, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_EXPAND), 0, 0);

    label12 = gtk_label_new (_("G\305\202\303\263wne"));
    gtk_widget_show (label12);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 0), label12);

    table5 = gtk_table_new (6, 2, FALSE);
    gtk_widget_show (table5);
    gtk_container_add (GTK_CONTAINER (notebook), table5);
    gtk_container_set_border_width (GTK_CONTAINER (table5), 6);
    gtk_table_set_row_spacings (GTK_TABLE (table5), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table5), 12);

    label29 = gtk_label_new (_("Komunikat startowy:"));
    gtk_widget_show (label29);
    gtk_table_attach (GTK_TABLE (table5), label29, 0, 1, 5, 6, (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label29), 0, 0.5);

    initkierunek = gtk_combo_box_new_text ();
    gtk_widget_show (initkierunek);
    gtk_table_attach (GTK_TABLE (table5), initkierunek, 1, 2, 4, 5, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_combo_box_append_text (GTK_COMBO_BOX (initkierunek), _("Automatyczny"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (initkierunek), _("Zach\303\263d"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (initkierunek), _("P\303\263\305\202noc"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (initkierunek), _("Wsch\303\263d"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (initkierunek), _("Po\305\202udnie"));

    initfontbutton = gtk_font_button_new ();
    gtk_widget_show (initfontbutton);
    gtk_table_attach (GTK_TABLE (table5), initfontbutton, 1, 2, 0, 1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_font_button_set_title (GTK_FONT_BUTTON (initfontbutton), _("Wyb\303\263r czcionki komunikau startowego"));
    gtk_font_button_set_use_font (GTK_FONT_BUTTON (initfontbutton), TRUE);
    gtk_font_button_set_use_size (GTK_FONT_BUTTON (initfontbutton), TRUE);

    label25 = gtk_label_new (_("Czcionka:"));
    gtk_widget_show (label25);
    gtk_table_attach (GTK_TABLE (table5), label25, 0, 1, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label25), 0, 0.5);

    label28 = gtk_label_new (_("<b>Kolor</b>"));
    gtk_widget_show (label28);
    gtk_table_attach (GTK_TABLE (table5), label28, 0, 2, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_label_set_use_markup (GTK_LABEL (label28), TRUE);
    gtk_misc_set_alignment (GTK_MISC (label28), 0, 0.5);

    alignment10 = gtk_alignment_new (0.5, 0.5, 1, 1);
    gtk_widget_show (alignment10);
    gtk_table_attach (GTK_TABLE (table5), alignment10, 0, 1, 2, 3, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_alignment_set_padding (GTK_ALIGNMENT (alignment10), 0, 0, 12, 0);

    label26 = gtk_label_new (_("Tekstu:"));
    gtk_widget_show (label26);
    gtk_container_add (GTK_CONTAINER (alignment10), label26);
    gtk_misc_set_alignment (GTK_MISC (label26), 0, 0.5);

    alignment11 = gtk_alignment_new (0.5, 0.5, 1, 1);
    gtk_widget_show (alignment11);
    gtk_table_attach (GTK_TABLE (table5), alignment11, 0, 1, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_alignment_set_padding (GTK_ALIGNMENT (alignment11), 0, 0, 12, 0);

    label27 = gtk_label_new (_("T\305\202a:"));
    gtk_widget_show (label27);
    gtk_container_add (GTK_CONTAINER (alignment11), label27);
    gtk_misc_set_alignment (GTK_MISC (label27), 0, 0.5);

    label36 = gtk_label_new (_("Kierunek liter:"));
    gtk_widget_show (label36);
    gtk_table_attach (GTK_TABLE (table5), label36, 0, 1, 4, 5, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label36), 0, 0.5);

    initfgcolorbutton = gtk_color_button_new ();
    gtk_widget_show (initfgcolorbutton);
    gtk_table_attach (GTK_TABLE (table5), initfgcolorbutton, 1, 2, 2, 3, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_color_button_set_title (GTK_COLOR_BUTTON (initfgcolorbutton), _("Wyb\303\263r koloru komunikatu startowego (tekst)"));

    initbgcolorbutton = gtk_color_button_new ();
    gtk_widget_show (initbgcolorbutton);
    gtk_table_attach (GTK_TABLE (table5), initbgcolorbutton, 1, 2, 3, 4, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_color_button_set_title (GTK_COLOR_BUTTON (initbgcolorbutton), _("Wyb\303\263r koloru komunikatu startowego (t\305\202o)"));

    inittext = gtk_entry_new ();
    gtk_widget_show (inittext);
    gtk_table_attach (GTK_TABLE (table5), inittext, 1, 2, 5, 6, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_widget_set_events (inittext, GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
    gtk_entry_set_invisible_char (GTK_ENTRY (inittext), 9679);

    label7 = gtk_label_new (_("Start"));
    gtk_widget_show (label7);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 1), label7);

    table7 = gtk_table_new (5, 2, FALSE);
    gtk_widget_show (table7);
    gtk_container_add (GTK_CONTAINER (notebook), table7);
    gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (notebook), table7, FALSE, FALSE, GTK_PACK_START);
    gtk_container_set_border_width (GTK_CONTAINER (table7), 6);
    gtk_table_set_row_spacings (GTK_TABLE (table7), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table7), 12);

    pilnefontbutton = gtk_font_button_new ();
    gtk_widget_show (pilnefontbutton);
    gtk_table_attach (GTK_TABLE (table7), pilnefontbutton, 1, 2, 0, 1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_font_button_set_title (GTK_FONT_BUTTON (pilnefontbutton), _("Wyb\303\263r czcionki PILNE"));
    gtk_font_button_set_use_font (GTK_FONT_BUTTON (pilnefontbutton), TRUE);
    gtk_font_button_set_use_size (GTK_FONT_BUTTON (pilnefontbutton), TRUE);

    pilnekierunek = gtk_combo_box_new_text ();
    gtk_widget_show (pilnekierunek);
    gtk_table_attach (GTK_TABLE (table7), pilnekierunek, 1, 2, 4, 5, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_combo_box_append_text (GTK_COMBO_BOX (pilnekierunek), _("Automatyczny"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (pilnekierunek), _("Zach\303\263d"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (pilnekierunek), _("P\303\263\305\202noc"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (pilnekierunek), _("Wsch\303\263d"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (pilnekierunek), _("Po\305\202udnie"));

    label30 = gtk_label_new (_("Czcionka:"));
    gtk_widget_show (label30);
    gtk_table_attach (GTK_TABLE (table7), label30, 0, 1, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label30), 0, 0.5);

    label33 = gtk_label_new (_("<b>Kolor</b>"));
    gtk_widget_show (label33);
    gtk_table_attach (GTK_TABLE (table7), label33, 0, 2, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_label_set_use_markup (GTK_LABEL (label33), TRUE);
    gtk_misc_set_alignment (GTK_MISC (label33), 0, 0.5);

    alignment12 = gtk_alignment_new (0.5, 0.5, 1, 1);
    gtk_widget_show (alignment12);
    gtk_table_attach (GTK_TABLE (table7), alignment12, 0, 1, 2, 3, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_alignment_set_padding (GTK_ALIGNMENT (alignment12), 0, 0, 12, 0);

    label31 = gtk_label_new (_("Tekstu:"));
    gtk_widget_show (label31);
    gtk_container_add (GTK_CONTAINER (alignment12), label31);
    gtk_misc_set_alignment (GTK_MISC (label31), 0, 0.5);

    alignment13 = gtk_alignment_new (0.5, 0.5, 1, 1);
    gtk_widget_show (alignment13);
    gtk_table_attach (GTK_TABLE (table7), alignment13, 0, 1, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_alignment_set_padding (GTK_ALIGNMENT (alignment13), 0, 0, 12, 0);

    label32 = gtk_label_new (_("T\305\202a:"));
    gtk_widget_show (label32);
    gtk_container_add (GTK_CONTAINER (alignment13), label32);
    gtk_misc_set_alignment (GTK_MISC (label32), 0, 0.5);

    label5 = gtk_label_new (_("Kierunek liter:"));
    gtk_widget_show (label5);
    gtk_table_attach (GTK_TABLE (table7), label5, 0, 1, 4, 5, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);

    pilnebgcolorbutton = gtk_color_button_new ();
    gtk_widget_show (pilnebgcolorbutton);
    gtk_table_attach (GTK_TABLE (table7), pilnebgcolorbutton, 1, 2, 3, 4, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_color_button_set_title (GTK_COLOR_BUTTON (pilnebgcolorbutton), _("Wyb\303\263r koloru PILNE (t\305\202o)"));

    pilnefgcolorbutton = gtk_color_button_new ();
    gtk_widget_show (pilnefgcolorbutton);
    gtk_table_attach (GTK_TABLE (table7), pilnefgcolorbutton, 1, 2, 2, 3, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_color_button_set_title (GTK_COLOR_BUTTON (pilnefgcolorbutton), _("Wyb\303\263r koloru PILNE (tekst)"));

    label8 = gtk_label_new (_("PILNE"));
    gtk_widget_show (label8);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 2), label8);

    table4 = gtk_table_new (5, 2, FALSE);
    gtk_widget_show (table4);
    gtk_container_add (GTK_CONTAINER (notebook), table4);
    gtk_container_set_border_width (GTK_CONTAINER (table4), 6);
    gtk_table_set_row_spacings (GTK_TABLE (table4), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table4), 12);

    krajfontbutton = gtk_font_button_new ();
    gtk_widget_show (krajfontbutton);
    gtk_table_attach (GTK_TABLE (table4), krajfontbutton, 1, 2, 0, 1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_font_button_set_title (GTK_FONT_BUTTON (krajfontbutton), _("Wyb\303\263r czcionki KRAJ"));
    gtk_font_button_set_use_font (GTK_FONT_BUTTON (krajfontbutton), TRUE);
    gtk_font_button_set_use_size (GTK_FONT_BUTTON (krajfontbutton), TRUE);

    label18 = gtk_label_new (_("<b>Kolor</b>"));
    gtk_widget_show (label18);
    gtk_table_attach (GTK_TABLE (table4), label18, 0, 2, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_label_set_use_markup (GTK_LABEL (label18), TRUE);
    gtk_misc_set_alignment (GTK_MISC (label18), 0, 0.5);

    alignment6 = gtk_alignment_new (0.5, 0.5, 1, 1);
    gtk_widget_show (alignment6);
    gtk_table_attach (GTK_TABLE (table4), alignment6, 0, 1, 2, 3, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_alignment_set_padding (GTK_ALIGNMENT (alignment6), 0, 0, 12, 0);

    label19 = gtk_label_new (_("Tekstu:"));
    gtk_widget_show (label19);
    gtk_container_add (GTK_CONTAINER (alignment6), label19);
    gtk_misc_set_alignment (GTK_MISC (label19), 0, 0.5);

    alignment7 = gtk_alignment_new (0.5, 0.5, 1, 1);
    gtk_widget_show (alignment7);
    gtk_table_attach (GTK_TABLE (table4), alignment7, 0, 1, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_alignment_set_padding (GTK_ALIGNMENT (alignment7), 0, 0, 12, 0);

    label20 = gtk_label_new (_("T\305\202a:"));
    gtk_widget_show (label20);
    gtk_container_add (GTK_CONTAINER (alignment7), label20);
    gtk_misc_set_alignment (GTK_MISC (label20), 0, 0.5);

    label40 = gtk_label_new (_("Kierunek liter:"));
    gtk_widget_show (label40);
    gtk_table_attach (GTK_TABLE (table4), label40, 0, 1, 4, 5, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label40), 0, 0.5);

    krajkierunek = gtk_combo_box_new_text ();
    gtk_widget_show (krajkierunek);
    gtk_table_attach (GTK_TABLE (table4), krajkierunek, 1, 2, 4, 5, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_combo_box_append_text (GTK_COMBO_BOX (krajkierunek), _("Automatyczny"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (krajkierunek), _("Zach\303\263d"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (krajkierunek), _("P\303\263\305\202noc"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (krajkierunek), _("Wsch\303\263d"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (krajkierunek), _("Po\305\202udnie"));

    krajbgcolorbutton = gtk_color_button_new ();
    gtk_widget_show (krajbgcolorbutton);
    gtk_table_attach (GTK_TABLE (table4), krajbgcolorbutton, 1, 2, 3, 4, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_color_button_set_title (GTK_COLOR_BUTTON (krajbgcolorbutton), _("Wyb\303\263r koloru KRAJ (t\305\202o)"));

    krajfgcolorbutton = gtk_color_button_new ();
    gtk_widget_show (krajfgcolorbutton);
    gtk_table_attach (GTK_TABLE (table4), krajfgcolorbutton, 1, 2, 2, 3, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_color_button_set_title (GTK_COLOR_BUTTON (krajfgcolorbutton), _("Wyb\303\263r koloru KRAJ (tekst)"));

    label3 = gtk_label_new (_("Czcionka:"));
    gtk_widget_show (label3);
    gtk_table_attach (GTK_TABLE (table4), label3, 0, 1, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);

    labela = gtk_label_new (_("KRAJ"));
    gtk_widget_show (labela);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 3), labela);

    table6 = gtk_table_new (5, 2, FALSE);
    gtk_widget_show (table6);
    gtk_container_add (GTK_CONTAINER (notebook), table6);
    gtk_container_set_border_width (GTK_CONTAINER (table6), 6);
    gtk_table_set_row_spacings (GTK_TABLE (table6), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table6), 12);

    swiatkierunek = gtk_combo_box_new_text ();
    gtk_widget_show (swiatkierunek);
    gtk_table_attach (GTK_TABLE (table6), swiatkierunek, 1, 2, 4, 5, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_combo_box_append_text (GTK_COMBO_BOX (swiatkierunek), _("Automatyczny"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (swiatkierunek), _("Zach\303\263d"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (swiatkierunek), _("P\303\263\305\202noc"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (swiatkierunek), _("Wsch\303\263d"));
    gtk_combo_box_append_text (GTK_COMBO_BOX (swiatkierunek), _("Po\305\202udnie"));

    swiatfontbutton = gtk_font_button_new ();
    gtk_widget_show (swiatfontbutton);
    gtk_table_attach (GTK_TABLE (table6), swiatfontbutton, 1, 2, 0, 1, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_font_button_set_title (GTK_FONT_BUTTON (swiatfontbutton), _("Wyb\303\263r czcionki \305\232WIAT"));
    gtk_font_button_set_use_font (GTK_FONT_BUTTON (swiatfontbutton), TRUE);
    gtk_font_button_set_use_size (GTK_FONT_BUTTON (swiatfontbutton), TRUE);

    label21 = gtk_label_new (_("Czcionka:"));
    gtk_widget_show (label21);
    gtk_table_attach (GTK_TABLE (table6), label21, 0, 1, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label21), 0, 0.5);

    label22 = gtk_label_new (_("<b>Kolor</b>"));
    gtk_widget_show (label22);
    gtk_table_attach (GTK_TABLE (table6), label22, 0, 2, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_label_set_use_markup (GTK_LABEL (label22), TRUE);
    gtk_misc_set_alignment (GTK_MISC (label22), 0, 0.5);

    alignment8 = gtk_alignment_new (0.5, 0.5, 1, 1);
    gtk_widget_show (alignment8);
    gtk_table_attach (GTK_TABLE (table6), alignment8, 0, 1, 2, 3, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_alignment_set_padding (GTK_ALIGNMENT (alignment8), 0, 0, 12, 0);

    label23 = gtk_label_new (_("Tekstu:"));
    gtk_widget_show (label23);
    gtk_container_add (GTK_CONTAINER (alignment8), label23);
    gtk_misc_set_alignment (GTK_MISC (label23), 0, 0.5);

    alignment9 = gtk_alignment_new (0.5, 0.5, 1, 1);
    gtk_widget_show (alignment9);
    gtk_table_attach (GTK_TABLE (table6), alignment9, 0, 1, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_alignment_set_padding (GTK_ALIGNMENT (alignment9), 0, 0, 12, 0);

    label24 = gtk_label_new (_("T\305\202a:"));
    gtk_widget_show (label24);
    gtk_container_add (GTK_CONTAINER (alignment9), label24);
    gtk_misc_set_alignment (GTK_MISC (label24), 0, 0.5);

    label41 = gtk_label_new (_("Kierunek liter:"));
    gtk_widget_show (label41);
    gtk_table_attach (GTK_TABLE (table6), label41, 0, 1, 4, 5, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label41), 0, 0.5);

    swiatfgcolorbutton = gtk_color_button_new ();
    gtk_widget_show (swiatfgcolorbutton);
    gtk_table_attach (GTK_TABLE (table6), swiatfgcolorbutton, 1, 2, 2, 3, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_color_button_set_title (GTK_COLOR_BUTTON (swiatfgcolorbutton), _("Wyb\303\263r koloru \305\232WIAT (tekst)"));

    swiatbgcolorbutton = gtk_color_button_new ();
    gtk_widget_show (swiatbgcolorbutton);
    gtk_table_attach (GTK_TABLE (table6), swiatbgcolorbutton, 1, 2, 3, 4, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_color_button_set_title (GTK_COLOR_BUTTON (swiatbgcolorbutton), _("Wyb\303\263r koloru SWIAT (t\305\202o)"));

    label4 = gtk_label_new (_("\305\232WIAT"));
    gtk_widget_show (label4);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 4), label4);

    hbox2 = gtk_hbox_new (FALSE, 6);
    gtk_widget_show (hbox2);
    gtk_box_pack_start (GTK_BOX (vbox2), hbox2, TRUE, TRUE, 0);

    domyslnebutton = gtk_button_new_with_mnemonic (_("Przywr\303\263\304\207 domy\305\233lne ustawienia"));
    gtk_widget_show (domyslnebutton);
    gtk_box_pack_start (GTK_BOX (hbox2), domyslnebutton, FALSE, FALSE, 0);

    zamknijbutton = gtk_button_new ();
    gtk_widget_show (zamknijbutton);
    gtk_box_pack_end (GTK_BOX (hbox2), zamknijbutton, FALSE, FALSE, 0);
    GTK_WIDGET_SET_FLAGS (zamknijbutton, GTK_CAN_DEFAULT);

    alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
    gtk_widget_show (alignment1);
    gtk_container_add (GTK_CONTAINER (zamknijbutton), alignment1);

    hbox1 = gtk_hbox_new (FALSE, 2);
    gtk_widget_show (hbox1);
    gtk_container_add (GTK_CONTAINER (alignment1), hbox1);

    image1 = gtk_image_new_from_stock ("gtk-close", GTK_ICON_SIZE_BUTTON);
    gtk_widget_show (image1);
    gtk_box_pack_start (GTK_BOX (hbox1), image1, FALSE, FALSE, 0);

    labelb = gtk_label_new_with_mnemonic (_("_Zamknij"));
    gtk_widget_show (labelb);
    gtk_box_pack_start (GTK_BOX (hbox1), labelb, FALSE, FALSE, 0);

    g_signal_connect ((gpointer) pilnebgcolorbutton, "color_set", G_CALLBACK (on_pilnebgcolorbutton_color_set), (gpointer)pa);
    g_signal_connect ((gpointer) pilnefgcolorbutton, "color_set", G_CALLBACK (on_pilnefgcolorbutton_color_set), (gpointer)pa);
    g_signal_connect ((gpointer) pilnefontbutton, "font_set", G_CALLBACK (on_pilnefontbutton_font_set), (gpointer)pa);
    g_signal_connect ((gpointer) krajbgcolorbutton, "color_set", G_CALLBACK (on_krajbgcolorbutton_color_set), (gpointer)pa);
    g_signal_connect ((gpointer) krajfgcolorbutton, "color_set", G_CALLBACK (on_krajfgcolorbutton_color_set), (gpointer)pa);
    g_signal_connect ((gpointer) krajfontbutton, "font_set", G_CALLBACK (on_krajfontbutton_font_set), (gpointer)pa);
    g_signal_connect ((gpointer) swiatbgcolorbutton, "color_set", G_CALLBACK (on_swiatbgcolorbutton_color_set), (gpointer)pa);
    g_signal_connect ((gpointer) swiatfgcolorbutton, "color_set", G_CALLBACK (on_swiatfgcolorbutton_color_set), (gpointer)pa);
    g_signal_connect ((gpointer) swiatfontbutton, "font_set", G_CALLBACK (on_swiatfontbutton_font_set), (gpointer)pa);
    g_signal_connect ((gpointer) szerokoscspinbutton, "value_changed", G_CALLBACK (on_szerokoscspibutton_value_changed), (gpointer)pa);
    g_signal_connect ((gpointer) zamknijbutton, "clicked", G_CALLBACK (on_zamknijbutton_clicked), (gpointer)pa);
    g_signal_connect ((gpointer) preferences, "delete_event", G_CALLBACK (on_preferences_delete_event), (gpointer)pa);
    g_signal_connect ((gpointer) predkoschscale, "change_value", G_CALLBACK (on_szybkoschscale_change_value), (gpointer)pa);
    g_signal_connect ((gpointer) skokhscale, "change_value", G_CALLBACK (on_skokhscale_change_value), (gpointer)pa);
    g_signal_connect ((gpointer) initbgcolorbutton, "color_set", G_CALLBACK (on_initbgcolorbutton_color_set), (gpointer)pa);
    g_signal_connect ((gpointer) initfgcolorbutton, "color_set", G_CALLBACK (on_initfgcolorbutton_color_set), (gpointer)pa);
    g_signal_connect ((gpointer) initfontbutton, "font_set", G_CALLBACK (on_initfontbutton_font_set), (gpointer)pa);
    g_signal_connect ((gpointer) tlocolorbutton, "color_set", G_CALLBACK (on_tlocolorbutton_color_set), (gpointer)pa);
    g_signal_connect ((gpointer) pilnekierunek, "changed", G_CALLBACK (on_pilnekierunek_changed), (gpointer)pa);
    g_signal_connect ((gpointer) initkierunek, "changed", G_CALLBACK (on_initkierunek_changed), (gpointer)pa);
    g_signal_connect ((gpointer) krajkierunek, "changed", G_CALLBACK (on_krajkierunek_changed), (gpointer)pa);
    g_signal_connect ((gpointer) swiatkierunek, "changed", G_CALLBACK (on_swiatkierunek_changed), (gpointer)pa);
    g_signal_connect ((gpointer) inittext, "changed" ,G_CALLBACK (on_inittext_changed), (gpointer)pa);
    g_signal_connect ((gpointer) przeciagarkacheckbox, "toggled", G_CALLBACK (on_przeciagarka_toggled), (gpointer)pa);
    g_signal_connect ((gpointer) wypelnijcheckbox, "toggled", G_CALLBACK (on_wypelnij_toggled), (gpointer)pa);
    g_signal_connect ((gpointer) domyslnebutton, "clicked", G_CALLBACK (on_domyslnebutton_clicked), (gpointer)pa);

    /* dla lookup_widget(). */
    GLADE_HOOKUP_OBJECT_NO_REF (preferences, preferences, "preferences");
    GLADE_HOOKUP_OBJECT (preferences, vbox2, "vbox2");
    GLADE_HOOKUP_OBJECT (preferences, notebook, "notebook");
    GLADE_HOOKUP_OBJECT (preferences, table2, "table2");
    GLADE_HOOKUP_OBJECT (preferences, przeciagarkacheckbox, "przeciagarkacheckbox");
    GLADE_HOOKUP_OBJECT (preferences, wypelnijcheckbox, "wypelnijcheckbox");
    GLADE_HOOKUP_OBJECT (preferences, label34, "label34");
    GLADE_HOOKUP_OBJECT (preferences, label11, "label11");
    GLADE_HOOKUP_OBJECT (preferences, label35, "label35");
    GLADE_HOOKUP_OBJECT (preferences, label2, "label2");
    GLADE_HOOKUP_OBJECT (preferences, szerokoscspinbutton, "szerokoscspinbutton");
    GLADE_HOOKUP_OBJECT (preferences, predkoschscale, "predkoschscale");
    GLADE_HOOKUP_OBJECT (preferences, skokhscale, "skokhscale");
    GLADE_HOOKUP_OBJECT (preferences, tlocolorbutton, "tlocolorbutton");
    GLADE_HOOKUP_OBJECT (preferences, label12, "label12");
    GLADE_HOOKUP_OBJECT (preferences, table5, "table5");
    GLADE_HOOKUP_OBJECT (preferences, label29, "label29");
    GLADE_HOOKUP_OBJECT (preferences, initkierunek, "initkierunek");
    GLADE_HOOKUP_OBJECT (preferences, initfontbutton, "initfontbutton");
    GLADE_HOOKUP_OBJECT (preferences, label25, "label25");
    GLADE_HOOKUP_OBJECT (preferences, label28, "label28");
    GLADE_HOOKUP_OBJECT (preferences, alignment10, "alignment10");
    GLADE_HOOKUP_OBJECT (preferences, label26, "label26");
    GLADE_HOOKUP_OBJECT (preferences, alignment11, "alignment11");
    GLADE_HOOKUP_OBJECT (preferences, label27, "label27");
    GLADE_HOOKUP_OBJECT (preferences, label36, "label36");
    GLADE_HOOKUP_OBJECT (preferences, initfgcolorbutton, "initfgcolorbutton");
    GLADE_HOOKUP_OBJECT (preferences, initbgcolorbutton, "initbgcolorbutton");
    GLADE_HOOKUP_OBJECT (preferences, inittext, "inittext");
    GLADE_HOOKUP_OBJECT (preferences, label7, "label7");
    GLADE_HOOKUP_OBJECT (preferences, table7, "table7");
    GLADE_HOOKUP_OBJECT (preferences, pilnefontbutton, "pilnefontbutton");
    GLADE_HOOKUP_OBJECT (preferences, pilnekierunek, "pilnekierunek");
    GLADE_HOOKUP_OBJECT (preferences, label30, "label30");
    GLADE_HOOKUP_OBJECT (preferences, label33, "label33");
    GLADE_HOOKUP_OBJECT (preferences, alignment12, "alignment12");
    GLADE_HOOKUP_OBJECT (preferences, label31, "label31");
    GLADE_HOOKUP_OBJECT (preferences, alignment13, "alignment13");
    GLADE_HOOKUP_OBJECT (preferences, label32, "label32");
    GLADE_HOOKUP_OBJECT (preferences, label5, "label5");
    GLADE_HOOKUP_OBJECT (preferences, pilnebgcolorbutton, "pilnebgcolorbutton");
    GLADE_HOOKUP_OBJECT (preferences, pilnefgcolorbutton, "pilnefgcolorbutton");
    GLADE_HOOKUP_OBJECT (preferences, label8, "label8");
    GLADE_HOOKUP_OBJECT (preferences, table4, "table4");
    GLADE_HOOKUP_OBJECT (preferences, krajfontbutton, "krajfontbutton");
    GLADE_HOOKUP_OBJECT (preferences, label18, "label18");
    GLADE_HOOKUP_OBJECT (preferences, alignment6, "alignment6");
    GLADE_HOOKUP_OBJECT (preferences, label19, "label19");
    GLADE_HOOKUP_OBJECT (preferences, alignment7, "alignment7");
    GLADE_HOOKUP_OBJECT (preferences, label20, "label20");
    GLADE_HOOKUP_OBJECT (preferences, label40, "label40");
    GLADE_HOOKUP_OBJECT (preferences, krajkierunek, "krajkierunek");
    GLADE_HOOKUP_OBJECT (preferences, krajbgcolorbutton, "krajbgcolorbutton");
    GLADE_HOOKUP_OBJECT (preferences, krajfgcolorbutton, "krajfgcolorbutton");
    GLADE_HOOKUP_OBJECT (preferences, label3, "label3");
    GLADE_HOOKUP_OBJECT (preferences, labela, "labela");
    GLADE_HOOKUP_OBJECT (preferences, table6, "table6");
    GLADE_HOOKUP_OBJECT (preferences, swiatkierunek, "swiatkierunek");
    GLADE_HOOKUP_OBJECT (preferences, swiatfontbutton, "swiatfontbutton");
    GLADE_HOOKUP_OBJECT (preferences, label21, "label21");
    GLADE_HOOKUP_OBJECT (preferences, label22, "label22");
    GLADE_HOOKUP_OBJECT (preferences, alignment8, "alignment8");
    GLADE_HOOKUP_OBJECT (preferences, label23, "label23");
    GLADE_HOOKUP_OBJECT (preferences, alignment9, "alignment9");
    GLADE_HOOKUP_OBJECT (preferences, label24, "label24");
    GLADE_HOOKUP_OBJECT (preferences, label41, "label41");
    GLADE_HOOKUP_OBJECT (preferences, swiatfgcolorbutton, "swiatfgcolorbutton");
    GLADE_HOOKUP_OBJECT (preferences, swiatbgcolorbutton, "swiatbgcolorbutton");
    GLADE_HOOKUP_OBJECT (preferences, label4, "label4");
    GLADE_HOOKUP_OBJECT (preferences, hbox2, "hbox2");
    GLADE_HOOKUP_OBJECT (preferences, domyslnebutton, "domyslnebutton");
    GLADE_HOOKUP_OBJECT (preferences, zamknijbutton, "zamknijbutton");
    GLADE_HOOKUP_OBJECT (preferences, alignment1, "alignment1");
    GLADE_HOOKUP_OBJECT (preferences, hbox1, "hbox1");
    GLADE_HOOKUP_OBJECT (preferences, image1, "image1");
    GLADE_HOOKUP_OBJECT (preferences, labelb, "labelb");

    
    gtk_widget_grab_focus (zamknijbutton);
    gtk_widget_grab_default (zamknijbutton);

    return preferences;
}

/* ustawienia okna preferencji */
void preferences_set (PasekTVN24Private* pa)
{
    GtkWidget *widget = NULL;
    
    widget = lookup_widget (pa->preferences, "szerokoscspinbutton");
    gtk_spin_button_configure (GTK_SPIN_BUTTON (widget), GTK_ADJUSTMENT (gtk_adjustment_new (pa->scroll_size, 20, 9999, 1, 10, 10)), 1, 0);

    widget = lookup_widget (pa->preferences, "skokhscale");
    gtk_range_set_adjustment (GTK_RANGE (widget), GTK_ADJUSTMENT (gtk_adjustment_new ((gfloat)pa->skok, 0, 100, 1, 10, 10)));

    widget = lookup_widget (pa->preferences, "predkoschscale");
    gtk_range_set_adjustment (GTK_RANGE (widget), GTK_ADJUSTMENT (gtk_adjustment_new (33 - (gfloat)pa->speed, 2, 32, 1, 1, 0)));
    
    widget = lookup_widget (pa->preferences, "initfontbutton");
    gtk_font_button_set_font_name (GTK_FONT_BUTTON (widget), pa->init_font);

    widget = lookup_widget (pa->preferences, "pilnefontbutton");
    gtk_font_button_set_font_name (GTK_FONT_BUTTON (widget), pa->pilne_font);

    widget = lookup_widget (pa->preferences, "krajfontbutton");
    gtk_font_button_set_font_name (GTK_FONT_BUTTON (widget), pa->kraj_font);

    widget = lookup_widget (pa->preferences, "swiatfontbutton");
    gtk_font_button_set_font_name (GTK_FONT_BUTTON (widget), pa->swiat_font);

    widget = lookup_widget (pa->preferences, "initkierunek");
    gtk_combo_box_set_active (GTK_COMBO_BOX (widget), pa->init_kierunek);

    widget = lookup_widget (pa->preferences, "swiatkierunek");
    gtk_combo_box_set_active (GTK_COMBO_BOX (widget), pa->swiat_kierunek);

    widget = lookup_widget (pa->preferences, "pilnekierunek");
    gtk_combo_box_set_active (GTK_COMBO_BOX (widget), pa->pilne_kierunek);

    widget = lookup_widget (pa->preferences, "krajkierunek");
    gtk_combo_box_set_active (GTK_COMBO_BOX (widget), pa->kraj_kierunek);

    widget = lookup_widget (pa->preferences, "initbgcolorbutton");
    gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), pa->init_bg_color);

    widget = lookup_widget (pa->preferences, "initfgcolorbutton");
    gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), pa->init_fg_color);

    widget = lookup_widget (pa->preferences, "pilnefgcolorbutton");
    gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), pa->pilne_fg_color);

    widget = lookup_widget (pa->preferences, "pilnebgcolorbutton");
    gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), pa->pilne_bg_color);

    widget = lookup_widget (pa->preferences, "krajfgcolorbutton");
    gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), pa->kraj_fg_color);

    widget = lookup_widget (pa->preferences, "krajbgcolorbutton");
    gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), pa->kraj_bg_color);

    widget = lookup_widget (pa->preferences, "swiatfgcolorbutton");
    gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), pa->swiat_fg_color);

    widget = lookup_widget (pa->preferences, "swiatbgcolorbutton");
    gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), pa->swiat_bg_color);

    widget = lookup_widget (pa->preferences, "tlocolorbutton");
    gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), pa->tlo_color);

    widget = lookup_widget (pa->preferences, "inittext");
    gtk_entry_set_text (GTK_ENTRY (widget), pa->init_text);

    widget = lookup_widget (pa->preferences, "przeciagarkacheckbox");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), (gboolean)pa->przeciagarka);

    widget = lookup_widget (pa->preferences, "wypelnijcheckbox");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), (gboolean)pa->rozciagnij);
}

/* wyswietlenie dialogu preferencji */
void display_preferences_dialog (BonoboUIComponent *uic, gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    if (pa->preferences == NULL) pa->preferences = create_preferences(pa);
    
    preferences_set (pa);

    gtk_widget_show_all (pa->preferences);
}

/* otwarcie RSSa TVN24 w zewnętrznym programie */
void open_tvn24_rss (BonoboUIComponent *uic, gpointer user_data)
{
    gnome_url_show (TVN24_RSS_URL, NULL);
}

/* wysiwetlenie dialogu about */
void display_about_dialog (BonoboUIComponent *uic, gpointer user_data)
{
    gtk_show_about_dialog (NULL, "program-name", "Pasek TVN24", "version", VERSION, "comments", "Copyright \302\251 Adam Kłobukowski", (gchar *)NULL);
}

/* zresetowanie pixmapy */
void reset_pixmap (PasekTVN24Private* pa)
{
    if (pa->drawing_area->window)
    {
	    if (pa->pixmap) gdk_drawable_unref (pa->pixmap);

	    pa->pixmap = gdk_pixmap_new (pa->drawing_area->window, pa->drawing_area->allocation.width, pa->drawing_area->allocation.height, -1);
    }
}

/* zmiana kierunku z popapu na PANGO_GRAVITY */
gint direction (gint i)
{
    switch (i)
    {
    	case 0:
    	{
    	    return PANGO_GRAVITY_AUTO;
    	}
    	case 1:
    	{
    	    return PANGO_GRAVITY_NORTH;
    	}
    	case 2:
    	{
    	    return PANGO_GRAVITY_EAST;
    	}
    	case 3:
    	{
    	    return PANGO_GRAVITY_SOUTH;
    	}
    	case 4:
    	{
    	    return PANGO_GRAVITY_SOUTH;
    	}
    }

    return 0;
}

/* funkcja odrysowująca applet */
void draw (PasekTVN24Private* pa)
{
    GdkGC *gc = NULL;
    GdkGC *gc_bg = NULL;
    GdkGC *gc_tlo = NULL;
    GdkGC *gc_init = NULL;
    GdkGC *gc_swiat = NULL;
    GdkGC *gc_kraj = NULL;
    GdkGC *gc_pilne = NULL;
    GdkGC *gc_init_bg = NULL;
    GdkGC *gc_swiat_bg = NULL;
    GdkGC *gc_kraj_bg = NULL;
    GdkGC *gc_pilne_bg = NULL;

    gint i = 0;
    gint x = 0;
    gint width = 0;
    gint height = 0;
    struct CrawlEntry *ce = NULL;

    gc_init = gdk_gc_new (pa->drawing_area->window);
    gdk_gc_set_rgb_fg_color (gc_init, pa->init_fg_color);
    gdk_gc_set_rgb_bg_color (gc_init, pa->init_bg_color);

    gc_kraj = gdk_gc_new (pa->drawing_area->window);
    gdk_gc_set_rgb_fg_color (gc_kraj, pa->kraj_fg_color);
    gdk_gc_set_rgb_bg_color (gc_kraj, pa->kraj_bg_color);

    gc_swiat = gdk_gc_new (pa->drawing_area->window);
    gdk_gc_set_rgb_fg_color (gc_swiat, pa->swiat_fg_color);
    gdk_gc_set_rgb_bg_color (gc_swiat, pa->swiat_bg_color);

    gc_pilne = gdk_gc_new (pa->drawing_area->window);
    gdk_gc_set_rgb_fg_color (gc_pilne, pa->pilne_fg_color);
    gdk_gc_set_rgb_bg_color (gc_pilne, pa->pilne_bg_color);

    gc_init_bg = gdk_gc_new (pa->drawing_area->window);
    gdk_gc_set_rgb_fg_color (gc_init_bg, pa->init_bg_color);
    gdk_gc_set_rgb_bg_color (gc_init_bg, pa->init_bg_color);

    gc_kraj_bg = gdk_gc_new (pa->drawing_area->window);
    gdk_gc_set_rgb_fg_color (gc_kraj_bg, pa->kraj_bg_color);
    gdk_gc_set_rgb_bg_color (gc_kraj_bg, pa->kraj_bg_color);

    gc_swiat_bg = gdk_gc_new (pa->drawing_area->window);
    gdk_gc_set_rgb_fg_color (gc_swiat_bg, pa->swiat_bg_color);
    gdk_gc_set_rgb_bg_color (gc_swiat_bg, pa->swiat_bg_color);

    gc_pilne_bg = gdk_gc_new (pa->drawing_area->window);
    gdk_gc_set_rgb_fg_color (gc_pilne_bg, pa->pilne_bg_color);
    gdk_gc_set_rgb_bg_color (gc_pilne_bg, pa->pilne_bg_color);

    gc_tlo = gdk_gc_new (pa->drawing_area->window);
    gdk_gc_set_rgb_fg_color (gc_tlo, pa->tlo_color);
    gdk_gc_set_rgb_bg_color (gc_tlo, pa->tlo_color);

    gdk_draw_rectangle (pa->pixmap, gc_tlo, TRUE, 0, 0, pa->drawing_area->allocation.width, pa->drawing_area->allocation.height);

    x = pa->t;

    for (i = 0; i < pa->entries->len; i++)
    {
        ce = (struct CrawlEntry*)g_ptr_array_index (pa->entries, i);

        switch (ce->et)
        {
    	    case INIT:
    	    {
    	        gc = gc_init;
    	        gc_bg = gc_init_bg;
    	        break;
    	    }

    	    case KRAJ:
    	    {
    	        gc = gc_kraj;
    	        gc_bg = gc_kraj_bg;
    	        break;
    	    }

    	    case SWIAT:
    	    {
    	        gc = gc_swiat;
    	        gc_bg = gc_swiat_bg;
    	        break;
    	    }

    	    case PILNE:
    	    {
    	        gc = gc_pilne;
    	        gc_bg = gc_pilne_bg;
    	        break;
    	    }
    	}

/* width i height sa zawsze tak jakby tekst byl nowmalnie poziomo */
        pango_layout_get_size (ce->layout, &width, &height);

    	width = PANGO_PIXELS (width);
        height = PANGO_PIXELS (height);

    	if (pa->porient == PANEL_APPLET_ORIENT_UP || pa->porient == PANEL_APPLET_ORIENT_DOWN)
    	{
    	    if ((x + width > 0) && (x < pa->drawing_area->allocation.width) )
    	    {
        		gdk_draw_rectangle (pa->pixmap, gc_bg, TRUE, x, 0, width, pa->drawing_area->allocation.height);
    	        gdk_draw_layout (pa->pixmap, gc, x, (pa->drawing_area->allocation.height - height) >> 1, ce->layout);
    		
    	    }

    	    x += width;
    	}
    	else
    	{
    	    if ((x + width > 0) && (x < pa->drawing_area->allocation.height) )
    	    {
                    if (pa->porient == PANEL_APPLET_ORIENT_LEFT)
        	        {
        		    gdk_draw_rectangle (pa->pixmap, gc_bg, TRUE, 0, x, pa->drawing_area->allocation.width, width);
    	    	    gdk_draw_layout (pa->pixmap, gc, (pa->drawing_area->allocation.width - height) >> 1, x, ce->layout);
    		}
    	        else 
    		{
        		    gdk_draw_rectangle (pa->pixmap, gc_bg, TRUE, 0, pa->drawing_area->allocation.height - x - width, pa->drawing_area->allocation.width, width);
    	    	    gdk_draw_layout (pa->pixmap, gc, (pa->drawing_area->allocation.width - height) >> 1, pa->drawing_area->allocation.height - x - width, ce->layout);
    		}
    	    }

    	    x += width;
    	}
    }

    gdk_draw_drawable (pa->drawing_area->window, gc, pa->pixmap, 0, 0, 0, 0, -1, -1);
    gdk_window_process_updates (pa->drawing_area->window, TRUE);

    gdk_gc_unref (gc_tlo);
    gdk_gc_unref (gc_init);
    gdk_gc_unref (gc_kraj);
    gdk_gc_unref (gc_swiat);
    gdk_gc_unref (gc_pilne);
    gdk_gc_unref (gc_init_bg);
    gdk_gc_unref (gc_kraj_bg);
    gdk_gc_unref (gc_swiat_bg);
    gdk_gc_unref (gc_pilne_bg);
}

/* zwolnienie tablicy elementow */
void free_entries (GPtrArray* array)
{
    int i = 0;
    struct CrawlEntry* ce = NULL;

    for (i = 0; i < array->len; i++)
    {
    	ce = (struct CrawlEntry*)g_ptr_array_index (array, i);

    	g_free (ce->text);
    	ce->text = NULL;
    	if (ce->layout)
    	{
    	    g_object_unref (ce->layout);
    	    ce->layout = NULL;
    	}
    	g_slice_free (struct CrawlEntry, ce);
    }

    g_ptr_array_free (array, FALSE);
}

/* watek odrysowujacy applet */
gint draw_timer(gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;

    if (pa->pixmap)
    {
/* byla zmiana w konfiguracji */
    	if (pa->skok_t > pa->skok) pa->skok_t = pa->skok;

/* panel się zmniejszył */
        if (pa->porient == PANEL_APPLET_ORIENT_UP || pa->porient == PANEL_APPLET_ORIENT_DOWN)
        {
            if (pa->t > pa->drawing_area->allocation.width) pa->t = pa->drawing_area->allocation.width;
        }
        else
        {
            if (pa->t > pa->drawing_area->allocation.height) pa->t = pa->drawing_area->allocation.height;
        }

    	if (pa->skok_t < 0)
    	{
    	    draw (pa);

    	    pa->skok_t = pa->skok;
    	}

        pa->skok_t--;

        if (pa->t == G_MAXLONG) 
    	{
    	    if (pa->porient == PANEL_APPLET_ORIENT_UP || pa->porient == PANEL_APPLET_ORIENT_DOWN) pa->t = pa->drawing_area->allocation.width;
            else pa->t = pa->drawing_area->allocation.height;
    	}

        pa->t--;

        if (pa->t <= -pa->width -1)
        {
    	    if (pa->porient == PANEL_APPLET_ORIENT_UP || pa->porient == PANEL_APPLET_ORIENT_DOWN) pa->t = pa->drawing_area->allocation.width;
    	    else pa->t = pa->drawing_area->allocation.height;

    	    if (pa->new_entries)
    	    {
        		g_mutex_lock (pa->mutex);

        		free_entries (pa->entries);
        		pa->entries = pa->new_entries;
        		pa->new_entries = NULL;

        		g_mutex_unlock (pa->mutex);

    	        prepare_layouts (pa->entries, pa);
    	    }
    	}
    }

    return TRUE;
}

/* przygotowanie layoutow */
void prepare_layouts (GPtrArray* input, PasekTVN24Private* pa)
{
    gint i = 0;
    gint width = 0;
    gchar *text = NULL;
    gchar *font_desc = NULL;
    PangoContext* context = NULL;
    struct CrawlEntry *ce = NULL;
    PangoMatrix matrix_left = PANGO_MATRIX_INIT;
    PangoMatrix matrix_right = PANGO_MATRIX_INIT;
    PangoMatrix matrix_top = PANGO_MATRIX_INIT;
    gint dir = 0;

    pango_matrix_rotate (&matrix_left, 270.);
    pango_matrix_rotate (&matrix_right, 90.);

    pa->width = 0;

    for (i = 0; i < input->len; i++)
    {
    	ce = (struct CrawlEntry*)g_ptr_array_index (input, i);

    	switch (ce->et)
    	{
    	    case INIT:
    	    {
        		font_desc = pa->init_font;
        		dir = pa->init_kierunek;
        		break;
    	    }

    	    case KRAJ:
    	    {
        		font_desc = pa->kraj_font;
        		dir = pa->kraj_kierunek;
        		break;
    	    }

    	    case SWIAT:
    	    {
        		font_desc = pa->swiat_font;
        		dir = pa->swiat_kierunek;
        		break;
    	    }

    	    case PILNE:
    	    {
        		font_desc = pa->pilne_font;
        		dir = pa->pilne_kierunek;
        		break;
    	    }
    	}

    	text = g_strdup_printf ("<span font_desc=\"%s\">   %s   </span>", font_desc, ce->text);

    	if (ce->layout) g_object_unref(ce->layout);

        ce->layout = gtk_widget_create_pango_layout (pa->drawing_area, "");
    	context = pango_layout_get_context (ce->layout);
        pango_context_set_base_gravity (context, direction (dir));
        pango_context_set_gravity_hint (context, PANGO_GRAVITY_HINT_STRONG);

        if (pa->porient == PANEL_APPLET_ORIENT_UP || pa->porient == PANEL_APPLET_ORIENT_DOWN)
    	{
    	    pango_context_set_matrix (context, &matrix_top);
    	    pango_layout_set_alignment (ce->layout, PANGO_ALIGN_LEFT);
    	}

    	if (pa->porient == PANEL_APPLET_ORIENT_RIGHT)
    	{
    	    pango_layout_set_alignment (ce->layout, PANGO_ALIGN_CENTER);
    	    pango_context_set_matrix (context, &matrix_right);
    	}
            if (pa->porient == PANEL_APPLET_ORIENT_LEFT)
    	{
    	    pango_layout_set_alignment (ce->layout, PANGO_ALIGN_CENTER);
    	    pango_context_set_matrix (context, &matrix_left);
    	}

        pango_layout_set_markup (ce->layout, text, -1);

        pango_layout_get_size (ce->layout, &width, NULL);

        pa->width += PANGO_PIXELS (width);

    	g_free (text);
    }
}

/* odczytanie wartosci atrybutu */
gchar* get_attribute_value(gchar* name, xmlNode * node)
{
    xmlNode *cur_node = NULL;

    for (cur_node = node; cur_node; cur_node = cur_node->next) if (cur_node->type == XML_ATTRIBUTE_NODE) if (strcmp (name, (gchar*)cur_node->name) == 0) return (gchar*)cur_node->children->content;

    return NULL;
}

/* zanalizowanie xmla */
void analyse_xml(xmlNode *a_node, GPtrArray *output, PasekTVN24Private* pa)
{
    xmlNode *cur_node = NULL;
    gchar* name = NULL;
    gchar* text = NULL;
    gchar* nain = NULL; /* gcc warns about main :( */
    gchar* message = NULL;
    struct CrawlEntry* ce = NULL;

    gint et = KRAJ;

    for (cur_node = a_node; cur_node; cur_node = cur_node->next)
    {
	if (cur_node->type == XML_ELEMENT_NODE)
	{
	    if (strcmp ((gchar*)cur_node->name, "env") == 0)
	    {
    		name = get_attribute_value("name", ((xmlNode*)((xmlElement*)cur_node)->attributes));
    		text = get_attribute_value("Text", ((xmlNode*)((xmlElement*)cur_node)->attributes));
    		nain = get_attribute_value("main", ((xmlNode*)((xmlElement*)cur_node)->attributes));
    		message = get_attribute_value("Message", ((xmlNode*)((xmlElement*)cur_node)->attributes));

    		if (name)
    		{
    		    if (strcmp (name, "MESSAGE") == 0)
    		    {
        			if (message)
        			{
        			    et = KRAJ;

        			    if (strcmp (message, "PILNE") == 0) et = PILNE;
        			    if (strcmp (message, "KRAJ") == 0) et = KRAJ;
        			    if (strcmp (message, "SWIAT") == 0) et = SWIAT;
        			}
    		    }

    		    if (strcmp (name, "TEKST") == 0)
    		    {
    		        if (nain) if (strcmp (nain, "carousel") == 0)
        			{
        			    ce = g_slice_alloc (sizeof (struct CrawlEntry));

        			    ce->et = et;
        			    ce->text = g_strdup (text);
        			    ce->layout = NULL;

        			    g_ptr_array_add (output, (gpointer)ce);
    		        }
    		    }
    		}
	    }

        analyse_xml (cur_node->children, output, pa);
    	}
    }
}

/* sciagniecie xmla */
gpointer xml_reader(gpointer user_data)
{
    PasekTVN24Private* pa = (PasekTVN24Private*)user_data;
    gchar *buffer = NULL;
    GnomeVFSHandle   *handle;
    GnomeVFSResult    result;
    GnomeVFSFileSize bytes_read;	/* ilosc odczytanych bajtów */
    GnomeVFSFileInfo* info = NULL;
    gint toread = 0;			/* ilosc bajtow do odczytania */
    gchar *mesg = NULL;
    GTimeVal wait;
    GPtrArray *output = NULL;

    xmlDocPtr doc = NULL;
    xmlNode *element = NULL;

    gint i = 0;

    wait.tv_sec = 0;
    wait.tv_usec = 0;

    for (;;)
    {
        info = gnome_vfs_file_info_new ();
    	gnome_vfs_get_file_info(pa->uri, info, GNOME_VFS_FILE_INFO_DEFAULT);

        buffer = g_malloc(info->size * sizeof (gchar) + sizeof (gchar));

    	result = gnome_vfs_open (&handle, pa->uri, GNOME_VFS_OPEN_READ);

        toread = info->size;

    	while (toread != 0 && result == GNOME_VFS_OK)
        {
    	    result = gnome_vfs_read (handle, buffer + (info->size - toread), toread, &bytes_read);
    	    toread -= bytes_read;
    	}

    	buffer[info->size] = '\0';

    	gnome_vfs_close (handle);

        if (result == GNOME_VFS_OK)
    	{
    	    doc = xmlReadMemory (buffer, info->size, pa->uri, "UTF-8", XML_PARSE_NOERROR | XML_PARSE_NONET);

    	    if (doc)
    	    {
        		element = xmlDocGetRootElement(doc);

        		output = g_ptr_array_new ();
    	        analyse_xml (element, output, pa);

    	        if (output->len > 0)
        		{
        		    g_mutex_lock (pa->mutex);

        		    if (pa->new_entries) free_entries (pa->new_entries);

        		    pa->new_entries = output;

        		    g_mutex_unlock (pa->mutex);
        	    }
        	}

    	    xmlFreeDoc(doc);
    	    doc = NULL;
    	}

    	gnome_vfs_file_info_unref (info);
    	g_free (buffer);

    	g_get_current_time (&wait);
        g_time_val_add (&wait, 30 * 1000000);

    	mesg = (gchar*) g_async_queue_timed_pop (pa->queue, &wait);

    	if (mesg != NULL)
    	{
    	    if (strcmp (mesg, DIE_MESG) == 0) g_thread_exit (NULL);
    	}
    }
}

/* typ obiektu */
GType pasek_tvn24_get_type (void)
{
    static GType type = 0;
    if (type == 0)
    {
        static const GTypeInfo info = {
    	    sizeof (PasekTVN24Class),
    	    NULL,   /* base_init */
    	    NULL,   /* base_finalize */
    	    NULL,   /* class_init */
    	    NULL,   /* class_finalize */
    	    NULL,   /* class_data */
    	    sizeof (PasekTVN24),
        	0,      /* n_preallocs */
    	    pasek_tvn24_init
    	};

    	type = g_type_register_static (PANEL_TYPE_APPLET, "PasekTVN24Type", &info, 0);
    }

    return type;
}

/* wskaźnik na definicję klasy rodzica */
static GObjectClass *parent_class = NULL;

/* konstruktor instancji */
static void pasek_tvn24_init (GTypeInstance *instance, gpointer g_class)
{
    PasekTVN24 *self = (PasekTVN24 *)instance;

    GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);

    gobject_class->dispose = pasek_tvn24_dispose;
    gobject_class->finalize = pasek_tvn24_finalize;

/* inicjalizacja struktury appletu */
    self->priv = (PasekTVN24Private*)g_slice_alloc0(sizeof(PasekTVN24Private));

    self->priv->porient = panel_applet_get_orient(PANEL_APPLET (self));
    self->priv->entries = NULL;
    self->priv->new_entries = NULL;
    self->priv->width = 0;
    self->priv->height = 0;
    self->priv->panel_size = 0;
    self->priv->scroll_size = 300;
    self->priv->uri = g_strdup (TVN_URI);
    self->priv->init_text = g_strdup (INIT_TEXT);
    self->priv->drawing_area = NULL;
    self->priv->pixmap = NULL;
    self->priv->papplet = self;
    self->priv->preferences = NULL;
    self->priv->t = G_MAXLONG;
    self->priv->thread = NULL;
    self->priv->mutex = NULL;
    self->priv->queue = NULL;
    self->priv->init_font = g_strdup (START_FONT);
    self->priv->kraj_font = g_strdup (START_FONT);
    self->priv->swiat_font = g_strdup (START_FONT);
    self->priv->pilne_font = g_strdup (START_FONT);
    self->priv->init_fg_color = g_slice_alloc (sizeof (GdkColor));
    self->priv->init_bg_color = g_slice_alloc (sizeof (GdkColor));
    self->priv->kraj_fg_color = g_slice_alloc (sizeof (GdkColor));
    self->priv->kraj_bg_color = g_slice_alloc (sizeof (GdkColor));
    self->priv->swiat_fg_color = g_slice_alloc (sizeof (GdkColor));
    self->priv->swiat_bg_color = g_slice_alloc (sizeof (GdkColor));
    self->priv->pilne_fg_color = g_slice_alloc (sizeof (GdkColor));
    self->priv->pilne_bg_color = g_slice_alloc (sizeof (GdkColor));
    self->priv->tlo_color = g_slice_alloc (sizeof (GdkColor));
    self->priv->timeout = 0;
    self->priv->speed = 16;
    self->priv->skok = 0;
    self->priv->skok_t = 0;
    self->priv->init_kierunek = 0;
    self->priv->pilne_kierunek = 0;
    self->priv->kraj_kierunek = 0;
    self->priv->swiat_kierunek = 0;
    self->priv->rozciagnij = 0;
    self->priv->przeciagarka = 0;

    parent_class = g_type_class_peek_parent (g_class);
}

/* destruktor instancji */

/* dispose */
static void pasek_tvn24_dispose (GObject *obj)
{
    PasekTVN24 *self = (PasekTVN24 *)obj;

    if (self->priv)
    {
/* timer rysowania */
    	g_source_remove (self->priv->timeout);

/* komunikat śmierci do wątku */
    	g_async_queue_push (self->priv->queue, DIE_MESG);

/* zakończenie wątku (nie trzeba zwalniac pamieci) */
        g_thread_join (self->priv->thread);

/* zwalnianie pamieci */

    	if (self->priv->new_entries)
    	{
    	    free_entries (self->priv->new_entries);
    	    self->priv->new_entries = NULL;
    	}

    	if (self->priv->entries)
    	{
    	    free_entries (self->priv->entries);
    	    self->priv->entries = NULL;
    	}

    	if (self->priv->uri)
    	{
    	    g_free (self->priv->uri);
    	    self->priv->uri = NULL;
    	}

    	if (self->priv->init_text)
    	{
    	    g_free (self->priv->init_text);
    	    self->priv->init_text = NULL;
    	}

    	if (self->priv->drawing_area)
    	{
    	    gtk_widget_destroy (GTK_WIDGET (self->priv->drawing_area));
    	    self->priv->drawing_area = NULL;
    	}

    	if (self->priv->pixmap)
    	{
    	    g_object_unref (self->priv->pixmap);
    	    self->priv->pixmap = NULL;
    	}

    	if (self->priv->preferences)
    	{
    	    g_object_unref (self->priv->preferences);
    	    self->priv->preferences = NULL;
    	}

    	if (self->priv->mutex)
    	{
    	    g_mutex_free (self->priv->mutex);
    	    self->priv->mutex = NULL;
    	}

    	if (self->priv->queue)
    	{
    	    g_async_queue_unref (self->priv->queue);
    	    self->priv->queue = NULL;
    	}

    	if (self->priv->init_font)
    	{
    	    g_free (self->priv->init_font);
    	    self->priv->init_font = NULL;
    	}

    	if (self->priv->kraj_font)
    	{
    	    g_free (self->priv->kraj_font);
    	    self->priv->kraj_font = NULL;
    	}

    	if (self->priv->swiat_font)
    	{
    	    g_free (self->priv->swiat_font);
    	    self->priv->swiat_font = NULL;
    	}

    	if (self->priv->pilne_font)
    	{
    	    g_free (self->priv->pilne_font);
    	    self->priv->pilne_font = NULL;
    	}

    	if (self->priv->init_fg_color)
    	{
    	    g_slice_free (GdkColor, self->priv->init_fg_color);
    	    self->priv->init_fg_color = NULL;
    	}

    	if (self->priv->init_bg_color)
    	{
    	    g_slice_free (GdkColor, self->priv->init_bg_color);
    	    self->priv->init_bg_color = NULL;
    	}

    	if (self->priv->kraj_fg_color)
    	{
    	    g_slice_free (GdkColor, self->priv->kraj_fg_color);
    	    self->priv->kraj_fg_color = NULL;
    	}

    	if (self->priv->kraj_bg_color)
    	{
    	    g_slice_free (GdkColor, self->priv->kraj_bg_color);
    	    self->priv->kraj_bg_color = NULL;
    	}

    	if (self->priv->swiat_fg_color)
    	{
    	    g_slice_free (GdkColor, self->priv->swiat_fg_color);
    	    self->priv->swiat_fg_color = NULL;
    	}

    	if (self->priv->swiat_bg_color)
    	{
    	    g_slice_free (GdkColor, self->priv->swiat_bg_color);
    	    self->priv->swiat_bg_color = NULL;
    	}

    	if (self->priv->pilne_fg_color)
    	{
    	    g_slice_free (GdkColor, self->priv->pilne_fg_color);
    	    self->priv->pilne_fg_color = NULL;
    	}

    	if (self->priv->pilne_bg_color)
    	{
    	    g_slice_free (GdkColor, self->priv->pilne_bg_color);
    	    self->priv->pilne_bg_color= NULL;
    	}

    	if (self->priv->tlo_color)
    	{
    	    g_slice_free (GdkColor, self->priv->tlo_color);
    	    self->priv->tlo_color = NULL;
        }

    	g_slice_free (PasekTVN24Private, self->priv);
    	self->priv = NULL;
    }

/* do klasy nadrzędnej */
    G_OBJECT_CLASS (parent_class)->dispose (obj);
}

/* finalize */				
static void pasek_tvn24_finalize (GObject *obj)
{
/* do klasy nadrzędnej */
    G_OBJECT_CLASS (parent_class)->finalize (obj);
}

/* funkcja glowna */
gboolean panel_main (PasekTVN24 *applet, const gchar *iid, gpointer data)
{
    GtkWidget *box = NULL;
    struct CrawlEntry *ce = NULL;
    gchar *sval = NULL;
    gint ival = 0;
    GdkScreen *screen;
    GdkColormap *rgba;

    if (strcmp (iid, "OAFIID:PasekTVN24") != 0) return FALSE;

    PasekTVN24Private* pa = applet->priv;

/* odczytanie preferencji */
    panel_applet_add_preferences (PANEL_APPLET (applet), "/schemas/apps/tvn24/prefs", NULL);

    ival = panel_applet_gconf_get_int (PANEL_APPLET (applet), "size", NULL);

    if (ival == 0) pa->scroll_size = 300;
    else pa->scroll_size = ival;

    ival = panel_applet_gconf_get_int (PANEL_APPLET (applet), "speed", NULL);

    if (ival == 0) pa->speed = 16;
    else pa->speed = ival;

    pa->skok = panel_applet_gconf_get_int (PANEL_APPLET (applet), "skok", NULL);
    pa->init_kierunek = panel_applet_gconf_get_int (PANEL_APPLET (applet), "init_kierunek", NULL);
    pa->pilne_kierunek = panel_applet_gconf_get_int (PANEL_APPLET (applet), "pilne_kierunek", NULL);
    pa->kraj_kierunek = panel_applet_gconf_get_int (PANEL_APPLET (applet), "kraj_kierunek", NULL);
    pa->swiat_kierunek = panel_applet_gconf_get_int (PANEL_APPLET (applet), "swiat_kierunek", NULL);

    pa->rozciagnij = panel_applet_gconf_get_int (PANEL_APPLET (applet), "rozciagnij", NULL);
    pa->przeciagarka = panel_applet_gconf_get_int (PANEL_APPLET (applet), "przeciagarka", NULL);

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "kraj_font", NULL);
    if (sval)
    {
    	g_free (pa->kraj_font);
    	pa->kraj_font = sval;
    }
    sval = NULL;

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "swiat_font", NULL);
    if (sval)
    {
    	g_free (pa->swiat_font);
    	pa->swiat_font = sval;
    }
    sval = NULL;

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "pilne_font", NULL);
    if (sval)
    {
    	g_free (pa->pilne_font);
    	pa->pilne_font = sval;
    }
    sval = NULL;

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "init_font", NULL);
    if (sval)
    {
    	g_free (pa->init_font);
    	pa->init_font = sval;
    }
    sval = NULL;

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "init_text", NULL);
    if (sval)
    {
    	g_free (pa->init_text);
    	pa->init_text = sval;
    }
    sval = NULL;

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "kraj_fg_color", NULL);
    if (sval) gdk_color_parse (sval, pa->kraj_fg_color);
    else gdk_color_parse (FG_KRAJ, pa->kraj_fg_color);
    sval = NULL;

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "kraj_bg_color", NULL);
    if (sval) gdk_color_parse (sval, pa->kraj_bg_color);
    else gdk_color_parse (BG_KRAJ, pa->kraj_bg_color);
    sval = NULL;

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "swiat_fg_color", NULL);
    if (sval) gdk_color_parse (sval, pa->swiat_fg_color);
    else gdk_color_parse (FG_SWIAT, pa->swiat_fg_color);
    sval = NULL;

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "swiat_bg_color", NULL);
    if (sval) gdk_color_parse (sval, pa->swiat_bg_color);
    else gdk_color_parse (BG_SWIAT, pa->swiat_bg_color);
    sval = NULL;

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "pilne_fg_color", NULL);
    if (sval) gdk_color_parse (sval, pa->pilne_fg_color);
    else gdk_color_parse (FG_PILNE, pa->pilne_fg_color);
    sval = NULL;

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "pilne_bg_color", NULL);
    if (sval) gdk_color_parse (sval, pa->pilne_bg_color);
    else gdk_color_parse (BG_PILNE, pa->pilne_bg_color);
    sval = NULL;

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "init_fg_color", NULL);
    if (sval) gdk_color_parse (sval, pa->init_fg_color);
    else gdk_color_parse (FG_INIT, pa->init_fg_color);
    sval = NULL;

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "init_bg_color", NULL);
    if (sval) gdk_color_parse (sval, pa->init_bg_color);
    else gdk_color_parse (BG_INIT, pa->init_bg_color);
    sval = NULL;

    sval = panel_applet_gconf_get_string (PANEL_APPLET (applet), "tlo_color", NULL);
    if (sval) gdk_color_parse (sval, pa->tlo_color);
    else gdk_color_parse (BG_TLO, pa->tlo_color);
    sval = NULL;

/* miejsce do mazania */
    pa->drawing_area = gtk_drawing_area_new ();
    if (pa->porient == PANEL_APPLET_ORIENT_UP || pa->porient == PANEL_APPLET_ORIENT_DOWN) gtk_widget_set_size_request (GTK_WIDGET (pa->drawing_area), pa->scroll_size, panel_applet_get_size (PANEL_APPLET (applet)));
    else gtk_widget_set_size_request (GTK_WIDGET (pa->drawing_area), panel_applet_get_size (PANEL_APPLET (applet)), pa->scroll_size);

/* opcje panelu */
    ival = PANEL_APPLET_FLAGS_NONE;
    if (pa->rozciagnij) ival |= PANEL_APPLET_EXPAND_MAJOR;
    if (pa->przeciagarka) ival |= PANEL_APPLET_HAS_HANDLE;
    panel_applet_set_flags (PANEL_APPLET (applet), ival);

/* boxujemy */
    box = (GtkWidget*)gtk_event_box_new ();
    gtk_widget_add_events (GTK_WIDGET (pa->drawing_area), GDK_BUTTON_PRESS_MASK);
    gtk_container_add (GTK_CONTAINER (box), pa->drawing_area);
    gtk_container_set_border_width (GTK_CONTAINER (box), 0);

/* i na applet */
    gtk_container_add (GTK_CONTAINER (applet), box);

/* pokazujemy widgety */
    gtk_widget_show_all (GTK_WIDGET (applet));

    pa->panel_size = panel_applet_get_size (PANEL_APPLET (applet));

/* poczatkowy komunikat... */
    ce = g_slice_alloc (sizeof (struct CrawlEntry));
    ce->et = INIT;
    ce->text = g_strdup (pa->init_text);
    ce->layout = NULL;

    pa->entries = g_ptr_array_new ();
    g_ptr_array_add (pa->entries, ce);

/* podlaczamy sygnały */
    g_signal_connect (G_OBJECT (applet), "change-orient", G_CALLBACK (applet_orientation_change), (gpointer)pa);
    g_signal_connect (G_OBJECT (pa->drawing_area), "configure_event", G_CALLBACK (configure_event), (gpointer)pa);
    g_signal_connect (G_OBJECT (pa->drawing_area), "expose_event", G_CALLBACK (expose_event), (gpointer)pa);
    g_signal_connect (G_OBJECT (pa->drawing_area), "button-press-event", G_CALLBACK (drawable_on_button_press), (gpointer)pa);

/* menu kontekstiowe */
    panel_applet_setup_menu (PANEL_APPLET (applet), Context_menu_xml, menu_verbs, pa);

/* odpalamy timer rysowania */
    pa->timeout = g_timeout_add (pa->speed, draw_timer, (gpointer) pa);

/* odpalamy watek odczytywania danych */
    if (g_thread_supported () == FALSE) g_thread_init (NULL);

    pa->mutex = g_mutex_new ();
    pa->queue = g_async_queue_new ();
    pa->thread = g_thread_create (xml_reader, (gpointer)pa, TRUE, NULL);

    return TRUE;
}

PANEL_APPLET_BONOBO_FACTORY ("OAFIID:PasekTvn24_Factory", PASEKTVN24_TYPE_APPLET, "Pasek TVN24", VERSION, (PanelAppletFactoryCallback)panel_main, NULL);
