/*
 * Pasek TVN24 v0.2.8
 *
 * Copyright (C) 2007 Adam Klobukowski
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TVN24_H

#define TVN24_H

#include <panel-applet.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkimage.h>
#include <gtk/gtkbox.h>
#include <gtk/gtkdrawingarea.h>
#include <gtk/gtkspinbutton.h>
#include <gtk/gtkbutton.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <libgnomevfs/gnome-vfs.h>

#include <libxml/parser.h>
#include <libxml/tree.h>

/* sprawdzenie wersji GTK, starsze wersje nie maja gdk_color_to_string() */
#undef HAVE_GDK_COLOR_TO_STRING

#if GTK_CHECK_VERSION(2,11,0)
#define HAVE_GDK_COLOR_TO_STRING 1
#endif

/* komunikaty wymieniane pomiedzy watkami (chwilowo nieuzywane) */
#define DIE_MESG "DIE"

/* domyslne kolory */
#define BG_INIT "#0A3b6E"
#define FG_INIT "#FFFFFF"
#define BG_KRAJ "#0A3b6E"
#define FG_KRAJ "#FFFFFF"
#define BG_SWIAT "#0A3b6E"
#define FG_SWIAT "#FFFFFF"
#define BG_PILNE "#800000"
#define FG_PILNE "#FFFFFF"
#define BG_TLO "#0A3b6E"

/* domyslna czcionka */
#define START_FONT "Sans Bold 10"

/* domyślny adres feeda */
#define TVN_URI "http://www.tvn24.pl/messages.xml"

/* adres RSSa TVN24 */
#define TVN24_RSS_URL "http://www.tvn24.pl/rss_desktop.rss"

/* domyślny tekst startowy */
#define INIT_TEXT "Łączę się z serwerem..."

/* Definicja klasy */

#define PASEKTVN24_TYPE_APPLET		(pasek_tvn24_get_type ())

typedef struct _PasekTVN24 PasekTVN24;
typedef struct _PasekTVN24Class PasekTVN24Class;
typedef struct _PasekTVN24Private PasekTVN24Private;

struct _PasekTVN24 {
    PanelApplet parent;
    
    PasekTVN24Private* priv;    
};
    
struct _PasekTVN24Class {
    PanelAppletClass parent_class;
};

GType pasek_tvn24_get_type (void);
static void pasek_tvn24_init (GTypeInstance *instance, gpointer g_class);
static void pasek_tvn24_dispose (GObject *obj);
static void pasek_tvn24_finalize (GObject *obj);
static void pasek_tvn24_class_init (PasekTVN24Class *class);

/* Struktura appletu */
struct _PasekTVN24Private
{
    PanelAppletOrient porient;	/* orientacja panelu */
    GPtrArray *new_entries;	/* tekst do wyswietlenia w nastepnym przejsciu crawla */
    GPtrArray *entries;		/* aktualnie wyświetlany tekst */
    gint width;			/* szerokosc layoutu od pango */
    gint height;		/* wysokosc layoutu od pango */
    gint panel_size;		/* rozmiar panelu (szerokość/wysokość, zależnie od orienatacji) */
    gint scroll_size;		/* rozmiar miejsca skrolowanego */
    gchar *uri;			/* adres xmla */
    gchar *init_text;		/* tekst startowy */
    GtkWidget *drawing_area;	/* widget do rysowania */
    GdkPixmap *pixmap;		/* pixmap do rysowania */
    PasekTVN24 *papplet;	/* applet */
    GtkWidget *preferences;	/* okienko preferencji */
    glong t;			/* pozycja scrolla */
    GThread *thread;		/* watek odczytywania danych */
    GMutex *mutex;		/* mutex do synchronizacji */
    GAsyncQueue *queue;		/* kolejka do przesylania informacji do watku (chwilowo nieuzywane) */
    gchar *init_font;		/* czcionka poczatkowego komunikatu */
    gchar *kraj_font;		/* czcionka KRAJ */
    gchar *swiat_font;		/* czcionka SWIAT */
    gchar *pilne_font;		/* czcionka PILNE */
    GdkColor *init_fg_color;	/* kolor tekstu poczatkowego komunikatu */
    GdkColor *init_bg_color;	/* kolor tła poczatkowego komunikatu */
    GdkColor *kraj_fg_color;	/* kolor tekstu KRAJ */
    GdkColor *kraj_bg_color;	/* kolor tła KRAJ */
    GdkColor *swiat_fg_color;	/* kolor tekstu SWIAT */
    GdkColor *swiat_bg_color;	/* kolor tła SWIAT */
    GdkColor *pilne_fg_color;	/* kolor tekstu PILNE */
    GdkColor *pilne_bg_color;	/* kolor tła PILNE */
    GdkColor *tlo_color;	/* kolor tła */
    guint timeout;		/* numer zrodla timera rysowania */
    gint speed;			/* opóźnienie przewijania */
    gint skok;			/* skokowosc przewijania */
    gint skok_t;		/* skokowosc przewijania */
    gint init_kierunek;		/* kierunek liter komunikatu początkowego */
    gint pilne_kierunek;	/* kierunek liter PILNE */
    gint kraj_kierunek;		/* kierunek liter INIT */
    gint swiat_kierunek;	/* kierunek liter SWIAT */
    gint rozciagnij;		/* pasek wypelnia całe dostępne miejsce */
    gint przeciagarka;		/* łatwa w uzyciu przeciągarka */
};

/* typ elementu crawla */
enum EntryType
{
    INIT,
    KRAJ,
    SWIAT,
    PILNE
};

/* element crawla */
struct CrawlEntry
{
    gint et;
    gchar *text;
    PangoLayout *layout;
};

#ifndef HAVE_GDK_COLOR_TO_STRING
gchar* gdk_color_to_string (const GdkColor *color);
#endif

void on_domyslnebutton_clicked (GtkButton *button, gpointer user_data);
void on_przeciagarka_toggled (GtkToggleButton *togglebutton, gpointer user_data);
void on_wypelnij_toggled (GtkToggleButton *togglebutton, gpointer user_data);
void on_inittext_changed (GtkEditable *editable, gpointer user_data);
void on_krajkierunek_changed (GtkComboBox *combobox, gpointer user_data);
void on_swiatkierunek_changed (GtkComboBox *combobox, gpointer user_data);
void on_initkierunek_changed (GtkComboBox *combobox, gpointer user_data);
void on_pilnekierunek_changed (GtkComboBox *combobox, gpointer user_data);
void on_szerokoscspibutton_value_changed (GtkSpinButton *spinbutton, gpointer user_data);
void on_zamknijbutton_clicked (GtkButton *button, gpointer user_data);
gboolean on_preferences_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data);
gboolean on_szybkoschscale_change_value (GtkRange *range, GtkScrollType scroll, gdouble value, gpointer user_data);
gboolean on_skokhscale_change_value (GtkRange *range, GtkScrollType scroll, gdouble value, gpointer user_data);
void on_tlocolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data);
void on_pilnebgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data);
void on_initbgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data);
void on_krajbgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data);
void on_swiatbgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data);
void on_pilnefgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data);
void on_initfgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data);
void on_swiatfgcolorbutton_color_set (GtkColorButton *colorbutton, gpointer user_data);
void on_pilnefontbutton_font_set (GtkFontButton *fontbutton, gpointer user_data);
void on_initfontbutton_font_set (GtkFontButton *fontbutton, gpointer user_data);
void on_krajfontbutton_font_set (GtkFontButton *fontbutton, gpointer user_data);
void on_swiatfontbutton_font_set (GtkFontButton *fontbutton, gpointer user_data);
void on_krajfgcolorbutton_color_set (GtkColorButton  *colorbutton, gpointer user_data);
void applet_orientation_change (PanelApplet *panel, gint arg1, gpointer user_data);
gboolean configure_event (GtkWidget *widget, GdkEventConfigure *event, gpointer user_data);
gboolean expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer user_data);
gboolean drawable_on_button_press (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
GtkWidget* lookup_widget (GtkWidget *widget, const gchar *widget_name);
GtkWidget* create_preferences (PasekTVN24Private* pa);
void preferences_set (PasekTVN24Private* pa);
void display_preferences_dialog (BonoboUIComponent *uic, gpointer user_data);
void open_tvn24_rss (BonoboUIComponent *uic, gpointer user_data);
void display_about_dialog (BonoboUIComponent *uic, gpointer user_data);
void reset_pixmap (PasekTVN24Private* pa);
gint direction (gint i);
void draw (PasekTVN24Private* pa);
void free_entries (GPtrArray* array);
gint draw_timer(gpointer user_data);
void prepare_layouts (GPtrArray* input, PasekTVN24Private* pa);
gchar* get_attribute_value(gchar* name, xmlNode * node);
void analyse_xml(xmlNode *a_node, GPtrArray *output, PasekTVN24Private* pa);
gpointer xml_reader(gpointer user_data);
gboolean panel_main (PasekTVN24 *applet, const gchar *iid, gpointer data);

/* makra Glade */
#define GLADE_HOOKUP_OBJECT(component,widget,name) g_object_set_data_full (G_OBJECT (component), name, gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) g_object_set_data (G_OBJECT (component), name, widget)

#endif /* TVN24_H */

